@@ -4,17 +4,21 @@ mod build_plan;
 mod compilation;
 mod compile_kind;
 mod context;
+mod crate_type;
 mod custom_build;
 mod fingerprint;
 mod job;
 mod job_queue;
 mod layout;
 mod links;
+mod lto;
 mod output_depinfo;
+pub mod rustdoc;
 pub mod standard_lib;
 mod timings;
 mod unit;
 pub mod unit_dependencies;
+pub mod unit_graph;
 
 use std::env;
 use std::ffi::{OsStr, OsString};
@@ -28,27 +32,33 @@ use lazycell::LazyCell;
 use log::debug;
 
 pub use self::build_config::{BuildConfig, CompileMode, MessageFormat};
-pub use self::build_context::{BuildContext, FileFlavor, RustcTargetData, TargetInfo};
+pub use self::build_context::{BuildContext, FileFlavor, FileType, RustcTargetData, TargetInfo};
 use self::build_plan::BuildPlan;
 pub use self::compilation::{Compilation, Doctest};
 pub use self::compile_kind::{CompileKind, CompileTarget};
 pub use self::context::{Context, Metadata};
+pub use self::crate_type::CrateType;
 pub use self::custom_build::{BuildOutput, BuildScriptOutputs, BuildScripts};
 pub use self::job::Freshness;
 use self::job::{Job, Work};
 use self::job_queue::{JobQueue, JobState};
-pub use self::layout::is_bad_artifact_name;
+pub(crate) use self::layout::Layout;
+pub use self::lto::Lto;
 use self::output_depinfo::output_depinfo;
-use self::unit_dependencies::UnitDep;
+use self::unit_graph::UnitDep;
 pub use crate::core::compiler::unit::{Unit, UnitInterner};
+use crate::core::features::nightly_features_allowed;
 use crate::core::manifest::TargetSourcePath;
-use crate::core::profiles::{Lto, PanicStrategy, Profile};
-use crate::core::{Edition, Feature, InternedString, PackageId, Target};
+use crate::core::profiles::{PanicStrategy, Profile, Strip};
+use crate::core::{Edition, Feature, PackageId, Target};
 use crate::util::errors::{self, CargoResult, CargoResultExt, ProcessError, VerboseError};
+use crate::util::interning::InternedString;
 use crate::util::machine_message::Message;
 use crate::util::{self, machine_message, ProcessBuilder};
 use crate::util::{internal, join_paths, paths, profile};
 
+const RUSTDOC_CRATE_VERSION_FLAG: &str = "--crate-version";
+
 /// A glorified callback for executing calls to rustc. Rather than calling rustc
 /// directly, we'll use an `Executor`, giving clients an opportunity to intercept
 /// the build calls.
@@ -56,13 +66,13 @@ pub trait Executor: Send + Sync + 'static {
     /// Called after a rustc process invocation is prepared up-front for a given
     /// unit of work (may still be modified for runtime-known dependencies, when
     /// the work is actually executed).
-    fn init<'a, 'cfg>(&self, _cx: &Context<'a, 'cfg>, _unit: &Unit<'a>) {}
+    fn init(&self, _cx: &Context<'_, '_>, _unit: &Unit) {}
 
     /// In case of an `Err`, Cargo will not continue with the build process for
     /// this package.
     fn exec(
         &self,
-        cmd: ProcessBuilder,
+        cmd: &ProcessBuilder,
         id: PackageId,
         target: &Target,
         mode: CompileMode,
@@ -72,7 +82,7 @@ pub trait Executor: Send + Sync + 'static {
 
     /// Queried when queuing each unit of work. If it returns true, then the
     /// unit will always be rebuilt, independent of whether it needs to be.
-    fn force_rebuild(&self, _unit: &Unit<'_>) -> bool {
+    fn force_rebuild(&self, _unit: &Unit) -> bool {
         false
     }
 }
@@ -85,7 +95,7 @@ pub struct DefaultExecutor;
 impl Executor for DefaultExecutor {
     fn exec(
         &self,
-        cmd: ProcessBuilder,
+        cmd: &ProcessBuilder,
         _id: PackageId,
         _target: &Target,
         _mode: CompileMode,
@@ -97,17 +107,17 @@ impl Executor for DefaultExecutor {
     }
 }
 
-fn compile<'a, 'cfg: 'a>(
-    cx: &mut Context<'a, 'cfg>,
-    jobs: &mut JobQueue<'a, 'cfg>,
+fn compile<'cfg>(
+    cx: &mut Context<'_, 'cfg>,
+    jobs: &mut JobQueue<'cfg>,
     plan: &mut BuildPlan,
-    unit: &Unit<'a>,
+    unit: &Unit,
     exec: &Arc<dyn Executor>,
     force_rebuild: bool,
 ) -> CargoResult<()> {
     let bcx = cx.bcx;
     let build_plan = bcx.build_config.build_plan;
-    if !cx.compiled.insert(*unit) {
+    if !cx.compiled.insert(unit.clone()) {
         return Ok(());
     }
 
@@ -134,13 +144,13 @@ fn compile<'a, 'cfg: 'a>(
             };
             work.then(link_targets(cx, unit, false)?)
         } else {
-            let work = if cx.bcx.show_warnings(unit.pkg.package_id()) {
+            let work = if unit.show_warnings(bcx.config) {
                 replay_output_cache(
                     unit.pkg.package_id(),
-                    unit.target,
+                    &unit.target,
                     cx.files().message_cache_path(unit),
                     cx.bcx.build_config.message_format,
-                    cx.bcx.config.shell().supports_color(),
+                    cx.bcx.config.shell().err_supports_color(),
                 )
             } else {
                 Work::noop()
@@ -166,11 +176,7 @@ fn compile<'a, 'cfg: 'a>(
     Ok(())
 }
 
-fn rustc<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
-    exec: &Arc<dyn Executor>,
-) -> CargoResult<Work> {
+fn rustc(cx: &mut Context<'_, '_>, unit: &Unit, exec: &Arc<dyn Executor>) -> CargoResult<Work> {
     let mut rustc = prepare_rustc(cx, &unit.target.rustc_crate_types(), unit)?;
     let build_plan = cx.bcx.build_config.build_plan;
 
@@ -191,26 +197,21 @@ fn rustc<'a, 'cfg>(
     // don't pass the `-l` flags.
     let pass_l_flag = unit.target.is_lib() || !unit.pkg.targets().iter().any(|t| t.is_lib());
     let pass_cdylib_link_args = unit.target.is_cdylib();
-    let do_rename = unit.target.allows_underscores() && !unit.mode.is_any_test();
-    let real_name = unit.target.name().to_string();
-    let crate_name = unit.target.crate_name();
 
-    // Rely on `target_filenames` iterator as source of truth rather than rederiving filestem.
-    let rustc_dep_info_loc = if do_rename && cx.files().metadata(unit).is_none() {
-        root.join(&crate_name)
-    } else {
-        root.join(&cx.files().file_stem(unit))
-    }
-    .with_extension("d");
+    let dep_info_name = match cx.files().metadata(unit) {
+        Some(metadata) => format!("{}-{}.d", unit.target.crate_name(), metadata),
+        None => format!("{}.d", unit.target.crate_name()),
+    };
+    let rustc_dep_info_loc = root.join(dep_info_name);
     let dep_info_loc = fingerprint::dep_info_loc(cx, unit);
 
     rustc.args(cx.bcx.rustflags_args(unit));
     if cx.bcx.config.cli_unstable().binary_dep_depinfo {
-        rustc.arg("-Zbinary-dep-depinfo");
+        rustc.arg("-Z").arg("binary-dep-depinfo");
     }
     let mut output_options = OutputOptions::new(cx, unit);
     let package_id = unit.pkg.package_id();
-    let target = unit.target.clone();
+    let target = Target::clone(&unit.target);
     let mode = unit.mode;
 
     exec.init(cx, unit);
@@ -224,7 +225,8 @@ fn rustc<'a, 'cfg>(
         .unwrap_or_else(|| cx.bcx.config.cwd())
         .to_path_buf();
     let fingerprint_dir = cx.files().fingerprint_dir(unit);
-    let script_metadata = cx.find_build_script_metadata(*unit);
+    let script_metadata = cx.find_build_script_metadata(unit.clone());
+    let is_local = unit.is_local();
 
     return Ok(Work::new(move |state| {
         // Only at runtime have we discovered what the extra -L and -l
@@ -281,7 +283,7 @@ fn rustc<'a, 'cfg>(
             state.build_plan(buildkey, rustc.clone(), outputs.clone());
         } else {
             exec.exec(
-                rustc,
+                &rustc,
                 package_id,
                 &target,
                 mode,
@@ -292,20 +294,6 @@ fn rustc<'a, 'cfg>(
             .chain_err(|| format!("could not compile `{}`.", name))?;
         }
 
-        if do_rename && real_name != crate_name {
-            let dst = &outputs[0].path;
-            let src = dst.with_file_name(
-                dst.file_name()
-                    .unwrap()
-                    .to_str()
-                    .unwrap()
-                    .replace(&real_name, &crate_name),
-            );
-            if src.exists() && src.file_name() != dst.file_name() {
-                fs::rename(&src, &dst).chain_err(|| format!("could not rename crate {:?}", src))?;
-            }
-        }
-
         if rustc_dep_info_loc.exists() {
             fingerprint::translate_dep_info(
                 &rustc_dep_info_loc,
@@ -313,8 +301,9 @@ fn rustc<'a, 'cfg>(
                 &cwd,
                 &pkg_root,
                 &target_dir,
+                &rustc,
                 // Do not track source files in the fingerprint for registry dependencies.
-                current_id.source_id().is_path(),
+                is_local,
             )
             .chain_err(|| {
                 internal(format!(
@@ -322,8 +311,9 @@ fn rustc<'a, 'cfg>(
                     rustc_dep_info_loc.display()
                 ))
             })?;
-            debug!("rewinding mtime of {:?} to {}", dep_info_loc, timestamp);
-            filetime::set_file_times(dep_info_loc, timestamp, timestamp)?;
+            // This mtime shift allows Cargo to detect if a source file was
+            // modified in the middle of the build.
+            paths::set_file_time_no_err(dep_info_loc, timestamp);
         }
 
         Ok(())
@@ -392,11 +382,7 @@ fn rustc<'a, 'cfg>(
 
 /// Link the compiled target (often of form `foo-{metadata_hash}`) to the
 /// final target. This must happen during both "Fresh" and "Compile".
-fn link_targets<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
-    fresh: bool,
-) -> CargoResult<Work> {
+fn link_targets(cx: &mut Context<'_, '_>, unit: &Unit, fresh: bool) -> CargoResult<Work> {
     let bcx = cx.bcx;
     let outputs = cx.outputs(unit)?;
     let export_dir = cx.files().export_dir();
@@ -406,7 +392,7 @@ fn link_targets<'a, 'cfg>(
     let features = unit.features.iter().map(|s| s.to_string()).collect();
     let json_messages = bcx.build_config.emit_json();
     let executable = cx.get_executable(unit)?;
-    let mut target = unit.target.clone();
+    let mut target = Target::clone(&unit.target);
     if let TargetSourcePath::Metabuild = target.src_path() {
         // Give it something to serialize.
         let path = unit.pkg.manifest().metabuild_path(cx.bcx.ws.target_dir());
@@ -533,19 +519,22 @@ where
     search_path
 }
 
-fn prepare_rustc<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
-    crate_types: &[&str],
-    unit: &Unit<'a>,
+fn prepare_rustc(
+    cx: &mut Context<'_, '_>,
+    crate_types: &[CrateType],
+    unit: &Unit,
 ) -> CargoResult<ProcessBuilder> {
     let is_primary = cx.is_primary_package(unit);
+    let is_workspace = cx.bcx.ws.is_member(&unit.pkg);
 
-    let mut base = cx.compilation.rustc_process(unit.pkg, is_primary)?;
+    let mut base = cx
+        .compilation
+        .rustc_process(unit, is_primary, is_workspace)?;
     if cx.bcx.config.cli_unstable().jobserver_per_rustc {
         let client = cx.new_jobserver()?;
         base.inherit_jobserver(&client);
-        base.arg("-Zjobserver-token-requests");
-        assert!(cx.rustc_clients.insert(*unit, client).is_none());
+        base.arg("-Z").arg("jobserver-token-requests");
+        assert!(cx.rustc_clients.insert(unit.clone(), client).is_none());
     } else {
         base.inherit_jobserver(&cx.jobserver);
     }
@@ -554,12 +543,11 @@ fn prepare_rustc<'a, 'cfg>(
     Ok(base)
 }
 
-fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult<Work> {
+fn rustdoc(cx: &mut Context<'_, '_>, unit: &Unit) -> CargoResult<Work> {
     let bcx = cx.bcx;
-    let mut rustdoc = cx.compilation.rustdoc_process(unit.pkg, unit.target)?;
+    let mut rustdoc = cx.compilation.rustdoc_process(unit)?;
     rustdoc.inherit_jobserver(&cx.jobserver);
     rustdoc.arg("--crate-name").arg(&unit.target.crate_name());
-    add_crate_versions_if_requested(bcx, unit, &mut rustdoc);
     add_path_args(bcx, unit, &mut rustdoc);
     add_cap_lints(bcx, unit, &mut rustdoc);
 
@@ -582,21 +570,26 @@ fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult
 
     add_error_format_and_color(cx, &mut rustdoc, false)?;
 
-    if let Some(args) = bcx.extra_args_for(unit) {
+    if let Some(args) = cx.bcx.extra_args_for(unit) {
         rustdoc.args(args);
     }
 
     build_deps_args(&mut rustdoc, cx, unit)?;
+    rustdoc::add_root_urls(cx, unit, &mut rustdoc)?;
 
     rustdoc.args(bcx.rustdocflags_args(unit));
 
+    if !crate_version_flag_already_present(&rustdoc) {
+        append_crate_version_flag(unit, &mut rustdoc);
+    }
+
     let name = unit.pkg.name().to_string();
     let build_script_outputs = Arc::clone(&cx.build_script_outputs);
     let package_id = unit.pkg.package_id();
-    let target = unit.target.clone();
+    let target = Target::clone(&unit.target);
     let mut output_options = OutputOptions::new(cx, unit);
     let pkg_id = unit.pkg.package_id();
-    let script_metadata = cx.find_build_script_metadata(*unit);
+    let script_metadata = cx.find_build_script_metadata(unit.clone());
 
     Ok(Work::new(move |state| {
         if let Some(script_metadata) = script_metadata {
@@ -626,19 +619,19 @@ fn rustdoc<'a, 'cfg>(cx: &mut Context<'a, 'cfg>, unit: &Unit<'a>) -> CargoResult
     }))
 }
 
-fn add_crate_versions_if_requested(
-    bcx: &BuildContext<'_, '_>,
-    unit: &Unit<'_>,
-    rustdoc: &mut ProcessBuilder,
-) {
-    if !bcx.config.cli_unstable().crate_versions {
-        return;
-    }
+// The --crate-version flag could have already been passed in RUSTDOCFLAGS
+// or as an extra compiler argument for rustdoc
+fn crate_version_flag_already_present(rustdoc: &ProcessBuilder) -> bool {
+    rustdoc.get_args().iter().any(|flag| {
+        flag.to_str()
+            .map_or(false, |flag| flag.starts_with(RUSTDOC_CRATE_VERSION_FLAG))
+    })
+}
+
+fn append_crate_version_flag(unit: &Unit, rustdoc: &mut ProcessBuilder) {
     rustdoc
-        .arg("-Z")
-        .arg("unstable-options")
-        .arg("--crate-version")
-        .arg(&unit.pkg.version().to_string());
+        .arg(RUSTDOC_CRATE_VERSION_FLAG)
+        .arg(unit.pkg.version().to_string());
 }
 
 // The path that we pass to rustc is actually fairly important because it will
@@ -655,7 +648,7 @@ fn add_crate_versions_if_requested(
 //
 // The first returned value here is the argument to pass to rustc, and the
 // second is the cwd that rustc should operate in.
-fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>) -> (PathBuf, PathBuf) {
+fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit) -> (PathBuf, PathBuf) {
     let ws_root = bcx.ws.root();
     let src = match unit.target.src_path() {
         TargetSourcePath::Path(path) => path.to_path_buf(),
@@ -670,21 +663,21 @@ fn path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>) -> (PathBuf, PathBuf)
     (src, unit.pkg.root().to_path_buf())
 }
 
-fn add_path_args(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>, cmd: &mut ProcessBuilder) {
+fn add_path_args(bcx: &BuildContext<'_, '_>, unit: &Unit, cmd: &mut ProcessBuilder) {
     let (arg, cwd) = path_args(bcx, unit);
     cmd.arg(arg);
     cmd.cwd(cwd);
 }
 
-fn add_cap_lints(bcx: &BuildContext<'_, '_>, unit: &Unit<'_>, cmd: &mut ProcessBuilder) {
+fn add_cap_lints(bcx: &BuildContext<'_, '_>, unit: &Unit, cmd: &mut ProcessBuilder) {
     // If this is an upstream dep we don't want warnings from, turn off all
     // lints.
-    if !bcx.show_warnings(unit.pkg.package_id()) {
+    if !unit.show_warnings(bcx.config) {
         cmd.arg("--cap-lints").arg("allow");
 
     // If this is an upstream dep but we *do* want warnings, make sure that they
     // don't fail compilation.
-    } else if !unit.pkg.package_id().source_id().is_path() {
+    } else if !unit.is_local() {
         cmd.arg("--cap-lints").arg("warn");
     }
 }
@@ -709,6 +702,7 @@ fn add_error_format_and_color(
         // to emit a message that cargo will intercept.
         json.push_str(",artifacts");
     }
+
     match cx.bcx.build_config.message_format {
         MessageFormat::Short | MessageFormat::Json { short: true, .. } => {
             json.push_str(",diagnostic-short");
@@ -716,21 +710,40 @@ fn add_error_format_and_color(
         _ => {}
     }
     cmd.arg(json);
+
+    if nightly_features_allowed() {
+        let config = cx.bcx.config;
+        match (
+            config.cli_unstable().terminal_width,
+            config.shell().err_width().diagnostic_terminal_width(),
+        ) {
+            // Terminal width explicitly provided - only useful for testing.
+            (Some(Some(width)), _) => {
+                cmd.arg(format!("-Zterminal-width={}", width));
+            }
+            // Terminal width was not explicitly provided but flag was provided - common case.
+            (Some(None), Some(width)) => {
+                cmd.arg(format!("-Zterminal-width={}", width));
+            }
+            // User didn't opt-in.
+            _ => (),
+        }
+    }
+
     Ok(())
 }
 
-fn build_base_args<'a, 'cfg>(
-    cx: &mut Context<'a, 'cfg>,
+fn build_base_args(
+    cx: &mut Context<'_, '_>,
     cmd: &mut ProcessBuilder,
-    unit: &Unit<'a>,
-    crate_types: &[&str],
+    unit: &Unit,
+    crate_types: &[CrateType],
 ) -> CargoResult<()> {
     assert!(!unit.mode.is_run_custom_build());
 
     let bcx = cx.bcx;
     let Profile {
         ref opt_level,
-        ref lto,
         codegen_units,
         debuginfo,
         debug_assertions,
@@ -738,6 +751,7 @@ fn build_base_args<'a, 'cfg>(
         rpath,
         ref panic,
         incremental,
+        strip,
         ..
     } = unit.profile;
     let test = unit.mode.is_any_test();
@@ -754,7 +768,7 @@ fn build_base_args<'a, 'cfg>(
 
     if !test {
         for crate_type in crate_types.iter() {
-            cmd.arg("--crate-type").arg(crate_type);
+            cmd.arg("--crate-type").arg(crate_type.as_str());
         }
     }
 
@@ -770,7 +784,7 @@ fn build_base_args<'a, 'cfg>(
     }
 
     let prefer_dynamic = (unit.target.for_host() && !unit.target.is_custom_build())
-        || (crate_types.contains(&"dylib") && bcx.ws.members().any(|p| p != unit.pkg));
+        || (crate_types.contains(&CrateType::Dylib) && bcx.ws.members().any(|p| *p != unit.pkg));
     if prefer_dynamic {
         cmd.arg("-C").arg("prefer-dynamic");
     }
@@ -783,23 +797,9 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("-C").arg(format!("panic={}", panic));
     }
 
-    // Disable LTO for host builds as prefer_dynamic and it are mutually
-    // exclusive.
-    if unit.target.can_lto() && !unit.target.for_host() {
-        match *lto {
-            Lto::Bool(false) => {}
-            Lto::Bool(true) => {
-                cmd.args(&["-C", "lto"]);
-            }
-            Lto::Named(ref s) => {
-                cmd.arg("-C").arg(format!("lto={}", s));
-            }
-        }
-    }
+    cmd.args(&lto_args(cx, unit));
 
     if let Some(n) = codegen_units {
-        // There are some restrictions with LTO and codegen-units, so we
-        // only add codegen units when LTO is not used.
         cmd.arg("-C").arg(&format!("codegen-units={}", n));
     }
 
@@ -807,7 +807,7 @@ fn build_base_args<'a, 'cfg>(
         cmd.arg("-C").arg(format!("debuginfo={}", debuginfo));
     }
 
-    if let Some(args) = bcx.extra_args_for(unit) {
+    if let Some(args) = cx.bcx.extra_args_for(unit) {
         cmd.args(args);
     }
 
@@ -843,7 +843,7 @@ fn build_base_args<'a, 'cfg>(
         // will simply not be needed when the behavior is stabilized in the Rust
         // compiler itself.
         if *panic == PanicStrategy::Abort {
-            cmd.arg("-Zpanic-abort-tests");
+            cmd.arg("-Z").arg("panic-abort-tests");
         }
     } else if test {
         cmd.arg("--cfg").arg("test");
@@ -893,13 +893,18 @@ fn build_base_args<'a, 'cfg>(
         opt(cmd, "-C", "incremental=", Some(dir));
     }
 
+    if strip != Strip::None {
+        cmd.arg("-Z").arg(format!("strip={}", strip));
+    }
+
     if unit.is_std {
         // -Zforce-unstable-if-unmarked prevents the accidental use of
         // unstable crates within the sysroot (such as "extern crate libc" or
         // any non-public crate in the sysroot).
         //
         // RUSTC_BOOTSTRAP allows unstable features on stable.
-        cmd.arg("-Zforce-unstable-if-unmarked")
+        cmd.arg("-Z")
+            .arg("force-unstable-if-unmarked")
             .env("RUSTC_BOOTSTRAP", "1");
     }
 
@@ -922,10 +927,27 @@ fn build_base_args<'a, 'cfg>(
     Ok(())
 }
 
-fn build_deps_args<'a, 'cfg>(
+fn lto_args(cx: &Context<'_, '_>, unit: &Unit) -> Vec<OsString> {
+    let mut result = Vec::new();
+    let mut push = |arg: &str| {
+        result.push(OsString::from("-C"));
+        result.push(OsString::from(arg));
+    };
+    match cx.lto[unit] {
+        lto::Lto::Run(None) => push("lto"),
+        lto::Lto::Run(Some(s)) => push(&format!("lto={}", s)),
+        lto::Lto::Off => push("lto=off"),
+        lto::Lto::ObjectAndBitcode => {} // this is rustc's default
+        lto::Lto::OnlyBitcode => push("linker-plugin-lto"),
+        lto::Lto::OnlyObject => push("embed-bitcode=no"),
+    }
+    result
+}
+
+fn build_deps_args(
     cmd: &mut ProcessBuilder,
-    cx: &mut Context<'a, 'cfg>,
-    unit: &Unit<'a>,
+    cx: &mut Context<'_, '_>,
+    unit: &Unit,
 ) -> CargoResult<()> {
     let bcx = cx.bcx;
     cmd.arg("-L").arg(&{
@@ -951,7 +973,7 @@ fn build_deps_args<'a, 'cfg>(
     // error in the future (see PR #4797).
     if !deps
         .iter()
-        .any(|dep| !dep.unit.mode.is_doc() && dep.unit.target.linkable())
+        .any(|dep| !dep.unit.mode.is_doc() && dep.unit.target.is_linkable())
     {
         if let Some(dep) = deps
             .iter()
@@ -991,73 +1013,70 @@ fn build_deps_args<'a, 'cfg>(
 }
 
 /// Generates a list of `--extern` arguments.
-pub fn extern_args<'a>(
-    cx: &Context<'a, '_>,
-    unit: &Unit<'a>,
+pub fn extern_args(
+    cx: &Context<'_, '_>,
+    unit: &Unit,
     unstable_opts: &mut bool,
 ) -> CargoResult<Vec<OsString>> {
     let mut result = Vec::new();
     let deps = cx.unit_deps(unit);
 
     // Closure to add one dependency to `result`.
-    let mut link_to = |dep: &UnitDep<'a>,
-                       extern_crate_name: InternedString,
-                       noprelude: bool|
-     -> CargoResult<()> {
-        let mut value = OsString::new();
-        let mut opts = Vec::new();
-        if unit
-            .pkg
-            .manifest()
-            .features()
-            .require(Feature::public_dependency())
-            .is_ok()
-            && !dep.public
-        {
-            opts.push("priv");
-            *unstable_opts = true;
-        }
-        if noprelude {
-            opts.push("noprelude");
-            *unstable_opts = true;
-        }
-        if !opts.is_empty() {
-            value.push(opts.join(","));
-            value.push(":");
-        }
-        value.push(extern_crate_name.as_str());
-        value.push("=");
+    let mut link_to =
+        |dep: &UnitDep, extern_crate_name: InternedString, noprelude: bool| -> CargoResult<()> {
+            let mut value = OsString::new();
+            let mut opts = Vec::new();
+            if unit
+                .pkg
+                .manifest()
+                .features()
+                .require(Feature::public_dependency())
+                .is_ok()
+                && !dep.public
+            {
+                opts.push("priv");
+                *unstable_opts = true;
+            }
+            if noprelude {
+                opts.push("noprelude");
+                *unstable_opts = true;
+            }
+            if !opts.is_empty() {
+                value.push(opts.join(","));
+                value.push(":");
+            }
+            value.push(extern_crate_name.as_str());
+            value.push("=");
 
-        let mut pass = |file| {
-            let mut value = value.clone();
-            value.push(file);
-            result.push(OsString::from("--extern"));
-            result.push(value);
-        };
+            let mut pass = |file| {
+                let mut value = value.clone();
+                value.push(file);
+                result.push(OsString::from("--extern"));
+                result.push(value);
+            };
 
-        let outputs = cx.outputs(&dep.unit)?;
-        let mut outputs = outputs.iter().filter_map(|output| match output.flavor {
-            FileFlavor::Linkable { rmeta } => Some((output, rmeta)),
-            _ => None,
-        });
+            let outputs = cx.outputs(&dep.unit)?;
 
-        if cx.only_requires_rmeta(unit, &dep.unit) {
-            let (output, _rmeta) = outputs
-                .find(|(_output, rmeta)| *rmeta)
-                .expect("failed to find rlib dep for pipelined dep");
-            pass(&output.path);
-        } else {
-            for (output, rmeta) in outputs {
-                if !rmeta {
-                    pass(&output.path);
+            if cx.only_requires_rmeta(unit, &dep.unit) || dep.unit.mode.is_check() {
+                // Example: rlib dependency for an rlib, rmeta is all that is required.
+                let output = outputs
+                    .iter()
+                    .find(|output| output.flavor == FileFlavor::Rmeta)
+                    .expect("failed to find rmeta dep for pipelined dep");
+                pass(&output.path);
+            } else {
+                // Example: a bin needs `rlib` for dependencies, it cannot use rmeta.
+                for output in outputs.iter() {
+                    if output.flavor == FileFlavor::Linkable {
+                        pass(&output.path);
+                    }
                 }
             }
-        }
-        Ok(())
-    };
+            Ok(())
+        };
 
     for dep in deps {
-        if dep.unit.target.linkable() && !dep.unit.mode.is_doc() {
+        if dep.unit.target.is_linkable() && !dep.unit.mode.is_doc() {
             link_to(dep, dep.extern_crate_name, dep.noprelude)?;
         }
     }
@@ -1093,9 +1112,9 @@ struct OutputOptions {
 }
 
 impl OutputOptions {
-    fn new<'a>(cx: &Context<'a, '_>, unit: &Unit<'a>) -> OutputOptions {
+    fn new(cx: &Context<'_, '_>, unit: &Unit) -> OutputOptions {
         let look_for_metadata_directive = cx.rmeta_required(unit);
-        let color = cx.bcx.config.shell().supports_color();
+        let color = cx.bcx.config.shell().err_supports_color();
         let path = cx.files().message_cache_path(unit);
         // Remove old cache, ignore ENOENT, which is the common case.
         drop(fs::remove_file(&path));
@@ -1130,7 +1149,7 @@ fn on_stderr_line(
         // Check if caching is enabled.
         if let Some((path, cell)) = &mut options.cache_cell {
             // Cache the output, which will be replayed later when Fresh.
-            let f = cell.try_borrow_mut_with(|| File::create(path))?;
+            let f = cell.try_borrow_mut_with(|| paths::create(path))?;
             debug_assert!(!line.contains('\n'));
             f.write_all(line.as_bytes())?;
             f.write_all(&[b'\n'])?;
@@ -1318,7 +1337,7 @@ fn replay_output_cache(
         // We sometimes have gigabytes of output from the compiler, so avoid
         // loading it all into memory at once, as that can cause OOM where
         // otherwise there would be none.
-        let file = fs::File::open(&path)?;
+        let file = paths::open(&path)?;
         let mut reader = std::io::BufReader::new(file);
         let mut line = String::new();
         loop {
