@@ -52,10 +52,25 @@ pub struct PublishOpts<'cfg> {
 
 pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
     let pkg = ws.current()?;
+    let mut publish_registry = opts.registry.clone();
 
     if let Some(ref allowed_registries) = *pkg.publish() {
-        let reg_name = opts
-            .registry
+        if publish_registry.is_none() && allowed_registries.len() == 1 {
+            // If there is only one allowed registry, push to that one directly,
+            // even though there is no registry specified in the command.
+            let default_registry = &allowed_registries[0];
+            if default_registry != CRATES_IO_REGISTRY {
+                // Don't change the registry for crates.io and don't warn the user.
+                // crates.io will be defaulted even without this.
+                opts.config.shell().note(&format!(
+                    "Found `{}` as only allowed registry. Publishing to it automatically.",
+                    default_registry
+                ))?;
+                publish_registry = Some(default_registry.clone());
+            }
+        }
+
+        let reg_name = publish_registry
             .clone()
             .unwrap_or_else(|| CRATES_IO_REGISTRY.to_string());
         if !allowed_registries.contains(&reg_name) {
@@ -72,7 +87,7 @@ pub fn publish(ws: &Workspace<'_>, opts: &PublishOpts<'_>) -> CargoResult<()> {
         opts.config,
         opts.token.clone(),
         opts.index.clone(),
-        opts.registry.clone(),
+        publish_registry,
         true,
         !opts.dry_run,
     )?;
@@ -233,10 +248,13 @@ fn transmit(
         ref badges,
         ref links,
     } = *manifest.metadata();
-    let readme_content = match *readme {
-        Some(ref readme) => Some(paths::read(&pkg.root().join(readme))?),
-        None => None,
-    };
+    let readme_content = readme
+        .as_ref()
+        .map(|readme| {
+            paths::read(&pkg.root().join(readme))
+                .chain_err(|| format!("failed to read `readme` file for package `{}`", pkg))
+        })
+        .transpose()?;
     if let Some(ref file) = *license_file {
         if !pkg.root().join(file).exists() {
             bail!("the license file `{}` does not exist", file)
@@ -662,7 +680,7 @@ pub fn registry_login(
         None => {
             drop_println!(
                 config,
-                "please visit {}/me and paste the API Token below",
+                "please paste the API Token found on {}/me below",
                 registry.host()
             );
             let mut line = String::new();
