@@ -1,29 +1,128 @@
+//! Utilities for handling git repositories, mainly around
+//! authentication/cloning.
+//!
+//! # `DefaultBranch` vs `Branch("master")`
+//!
+//! Long ago in a repository not so far away, an author (*cough* me *cough*)
+//! didn't understand how branches work in Git. This led the author to
+//! interpret these two dependency declarations the exact same way with the
+//! former literally internally desugaring to the latter:
+//!
+//! ```toml
+//! [dependencies]
+//! foo = { git = "https://example.org/foo" }
+//! foo = { git = "https://example.org/foo", branch = "master" }
+//! ```
+//!
+//! It turns out there's this things called `HEAD` in git remotes which points
+//! to the "main branch" of a repository, and the main branch is not always
+//! literally called master. What Cargo would like to do is to differentiate
+//! these two dependency directives, with the first meaning "depend on `HEAD`".
+//!
+//! Unfortunately implementing this is a breaking change. This was first
+//! attempted in #8364 but resulted in #8468 which has two independent bugs
+//! listed on that issue. Despite this breakage we would still like to roll out
+//! this change in Cargo, but we're now going to take it very slow and try to
+//! break as few people as possible along the way. These comments are intended
+//! to log the current progress and what wonkiness you might see within Cargo
+//! when handling `DefaultBranch` vs `Branch("master")`
+//!
+//! ### Repositories with `master` and a default branch
+//!
+//! This is one of the most obvious sources of breakage. If our `foo` example
+//! in above had two branches, one called `master` and another which was
+//! actually the main branch, then Cargo's change will always be a breaking
+//! change. This is because what's downloaded is an entirely different branch
+//! if we change the meaning of the dependency directive.
+//!
+//! It's expected this is quite rare, but to handle this case nonetheless when
+//! Cargo fetches from a git remote and the dependency specification is
+//! `DefaultBranch` then it will issue a warning if the `HEAD` reference
+//! doesn't match `master`. It's expected in this situation that authors will
+//! fix builds locally by specifying `branch = 'master'`.
+//!
+//! ### Differences in `cargo vendor` configuration
+//!
+//! When executing `cargo vendor` it will print out configuration which can
+//! then be used to configure Cargo to use the `vendor` directory. Historically
+//! this configuration looked like:
+//!
+//! ```toml
+//! [source."https://example.org/foo"]
+//! git = "https://example.org/foo"
+//! branch = "master"
+//! replace-with = "vendored-sources"
+//! ```
+//!
+//! We would like to, however, transition this to not include the `branch =
+//! "master"` unless the dependency directive actually mentions a branch.
+//! Conveniently older Cargo implementations all interpret a missing `branch`
+//! as `branch = "master"` so it's a backwards-compatible change to remove the
+//! `branch = "master"` directive. As a result, `cargo vendor` will no longer
+//! emit a `branch` if the git reference is `DefaultBranch`
+//!
+//! ### Differences in lock file formats
+//!
+//! Another issue pointed out in #8364 was that `Cargo.lock` files were no
+//! longer compatible on stable and nightly with each other. The underlying
+//! issue is that Cargo was serializing `branch = "master"` *differently* on
+//! nightly than it was on stable. Historical implementations of Cargo would
+//! encode `DefaultBranch` and `Branch("master")` the same way in `Cargo.lock`,
+//! so when reading a lock file we have no way of differentiating between the
+//! two.
+//!
+//! To handle this difference in encoding of `Cargo.lock` we'll be employing
+//! the standard scheme to change `Cargo.lock`:
+//!
+//! * Add support in Cargo for a future format, don't turn it on.
+//! * Wait a long time
+//! * Turn on the future format
+//!
+//! Here the "future format" is `branch=master` shows up if you have a `branch`
+//! in `Cargo.toml`, and otherwise nothing shows up in URLs. Due to the effect
+//! on crate graph resolution, however, this flows into the next point..
+//!
+//! ### Unification in the Cargo dependency graph
+//!
+//! Today dependencies with `branch = "master"` will unify with dependencies
+//! that say nothing. (that's because the latter simply desugars). This means
+//! the two `foo` directives above will resolve to the same dependency.
+//!
+//! The best idea I've got to fix this is to basically get everyone (if anyone)
+//! to stop doing this today. The crate graph resolver will start to warn if it
+//! detects that multiple `Cargo.toml` directives are detected and mixed.  The
+//! thinking is that when we turn on the new lock file format it'll also be
+//! hard breaking change for any project which still has dependencies to
+//! both the `master` branch and not.
+//!
+//! ### What we're doing today
+//!
+//! The general goal of Cargo today is to internally distinguish
+//! `DefaultBranch` and `Branch("master")`, but for the time being they should
+//! be functionally equivalent in terms of builds. The hope is that we'll let
+//! all these warnings and such bake for a good long time, and eventually we'll
+//! flip some switches if your build has no warnings it'll work before and
+//! after.
+//!
+//! That's the dream at least, we'll see how this plays out.
+
 use crate::core::GitReference;
 use crate::util::errors::{CargoResult, CargoResultExt};
 use crate::util::paths;
 use crate::util::process_builder::process;
 use crate::util::{network, Config, IntoUrl, Progress};
-use curl::easy::{Easy, List};
-use git2::{self, ObjectType};
+use anyhow::{anyhow, Context};
+use curl::easy::List;
+use git2::{self, ErrorClass, ObjectType};
 use log::{debug, info};
 use serde::ser;
 use serde::Serialize;
 use std::env;
 use std::fmt;
-use std::fs::File;
 use std::path::{Path, PathBuf};
 use std::process::Command;
 use url::Url;
 
-#[derive(PartialEq, Clone, Debug)]
-pub struct GitRevision(git2::Oid);
-
-impl ser::Serialize for GitRevision {
-    fn serialize<S: ser::Serializer>(&self, s: S) -> Result<S::Ok, S::Error> {
-        serialize_str(self, s)
-    }
-}
-
 fn serialize_str<T, S>(t: &T, s: S) -> Result<S::Ok, S::Error>
 where
     T: fmt::Display,
@@ -32,12 +131,6 @@ where
     s.collect_str(t)
 }
 
-impl fmt::Display for GitRevision {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        fmt::Display::fmt(&self.0, f)
-    }
-}
-
 pub struct GitShortID(git2::Buf);
 
 impl GitShortID {
@@ -71,7 +164,8 @@ pub struct GitDatabase {
 pub struct GitCheckout<'a> {
     database: &'a GitDatabase,
     location: PathBuf,
-    revision: GitRevision,
+    #[serde(serialize_with = "serialize_str")]
+    revision: git2::Oid,
     #[serde(skip_serializing)]
     repo: git2::Repository,
 }
@@ -87,33 +181,52 @@ impl GitRemote {
         &self.url
     }
 
-    pub fn rev_for(&self, path: &Path, reference: &GitReference) -> CargoResult<GitRevision> {
-        reference.resolve(&self.db_at(path)?.repo)
+    pub fn rev_for(&self, path: &Path, reference: &GitReference) -> CargoResult<git2::Oid> {
+        reference.resolve(&self.db_at(path)?.repo, None)
     }
 
     pub fn checkout(
         &self,
         into: &Path,
+        db: Option<GitDatabase>,
         reference: &GitReference,
+        locked_rev: Option<git2::Oid>,
         cargo_config: &Config,
-    ) -> CargoResult<(GitDatabase, GitRevision)> {
-        let mut repo_and_rev = None;
-        if let Ok(mut repo) = git2::Repository::open(into) {
-            self.fetch_into(&mut repo, cargo_config)
-                .chain_err(|| format!("failed to fetch into {}", into.display()))?;
-            if let Ok(rev) = reference.resolve(&repo) {
-                repo_and_rev = Some((repo, rev));
+    ) -> CargoResult<(GitDatabase, git2::Oid)> {
+        // If we have a previous instance of `GitDatabase` then fetch into that
+        // if we can. If that can successfully load our revision then we've
+        // populated the database with the latest version of `reference`, so
+        // return that database and the rev we resolve to.
+        if let Some(mut db) = db {
+            fetch(&mut db.repo, self.url.as_str(), reference, cargo_config)
+                .context(format!("failed to fetch into: {}", into.display()))?;
+            match locked_rev {
+                Some(rev) => {
+                    if db.contains(rev) {
+                        return Ok((db, rev));
+                    }
+                }
+                None => {
+                    if let Ok(rev) = reference.resolve(&db.repo, Some((&self.url, cargo_config))) {
+                        return Ok((db, rev));
+                    }
+                }
             }
         }
-        let (repo, rev) = match repo_and_rev {
-            Some(pair) => pair,
-            None => {
-                let repo = self
-                    .clone_into(into, cargo_config)
-                    .chain_err(|| format!("failed to clone into: {}", into.display()))?;
-                let rev = reference.resolve(&repo)?;
-                (repo, rev)
-            }
+
+        // Otherwise start from scratch to handle corrupt git repositories.
+        // After our fetch (which is interpreted as a clone now) we do the same
+        // resolution to figure out what we cloned.
+        if into.exists() {
+            paths::remove_dir_all(into)?;
+        }
+        paths::create_dir_all(into)?;
+        let mut repo = init(into, true)?;
+        fetch(&mut repo, self.url.as_str(), reference, cargo_config)
+            .context(format!("failed to clone into: {}", into.display()))?;
+        let rev = match locked_rev {
+            Some(rev) => rev,
+            None => reference.resolve(&repo, Some((&self.url, cargo_config)))?,
         };
 
         Ok((
@@ -134,49 +247,32 @@ impl GitRemote {
             repo,
         })
     }
-
-    fn fetch_into(&self, dst: &mut git2::Repository, cargo_config: &Config) -> CargoResult<()> {
-        // Create a local anonymous remote in the repository to fetch the url
-        let refspec = "refs/heads/*:refs/heads/*";
-        fetch(dst, self.url.as_str(), refspec, cargo_config)
-    }
-
-    fn clone_into(&self, dst: &Path, cargo_config: &Config) -> CargoResult<git2::Repository> {
-        if dst.exists() {
-            paths::remove_dir_all(dst)?;
-        }
-        paths::create_dir_all(dst)?;
-        let mut repo = init(dst, true)?;
-        fetch(
-            &mut repo,
-            self.url.as_str(),
-            "refs/heads/*:refs/heads/*",
-            cargo_config,
-        )?;
-        Ok(repo)
-    }
 }
 
 impl GitDatabase {
     pub fn copy_to(
         &self,
-        rev: GitRevision,
+        rev: git2::Oid,
         dest: &Path,
         cargo_config: &Config,
     ) -> CargoResult<GitCheckout<'_>> {
         let mut checkout = None;
         if let Ok(repo) = git2::Repository::open(dest) {
-            let mut co = GitCheckout::new(dest, self, rev.clone(), repo);
+            let mut co = GitCheckout::new(dest, self, rev, repo);
             if !co.is_fresh() {
-                // After a successful fetch operation do a sanity check to
-                // ensure we've got the object in our database to reset to. This
-                // can fail sometimes for corrupt repositories where the fetch
-                // operation succeeds but the object isn't actually there.
+                // After a successful fetch operation the subsequent reset can
+                // fail sometimes for corrupt repositories where the fetch
+                // operation succeeds but the object isn't actually there in one
+                // way or another. In these situations just skip the error and
+                // try blowing away the whole repository and trying with a
+                // clone.
                 co.fetch(cargo_config)?;
-                if co.has_object() {
-                    co.reset(cargo_config)?;
-                    assert!(co.is_fresh());
-                    checkout = Some(co);
+                match co.reset(cargo_config) {
+                    Ok(()) => {
+                        assert!(co.is_fresh());
+                        checkout = Some(co);
+                    }
+                    Err(e) => debug!("failed reset after fetch {:?}", e),
                 }
             } else {
                 checkout = Some(co);
@@ -190,37 +286,89 @@ impl GitDatabase {
         Ok(checkout)
     }
 
-    pub fn to_short_id(&self, revision: &GitRevision) -> CargoResult<GitShortID> {
-        let obj = self.repo.find_object(revision.0, None)?;
+    pub fn to_short_id(&self, revision: git2::Oid) -> CargoResult<GitShortID> {
+        let obj = self.repo.find_object(revision, None)?;
         Ok(GitShortID(obj.short_id()?))
     }
 
-    pub fn has_ref(&self, reference: &str) -> CargoResult<()> {
-        self.repo.revparse_single(reference)?;
-        Ok(())
+    pub fn contains(&self, oid: git2::Oid) -> bool {
+        self.repo.revparse_single(&oid.to_string()).is_ok()
+    }
+
+    pub fn resolve(
+        &self,
+        r: &GitReference,
+        remote_and_config: Option<(&Url, &Config)>,
+    ) -> CargoResult<git2::Oid> {
+        r.resolve(&self.repo, remote_and_config)
     }
 }
 
 impl GitReference {
-    fn resolve(&self, repo: &git2::Repository) -> CargoResult<GitRevision> {
-        let id = match *self {
-            GitReference::Tag(ref s) => (|| -> CargoResult<git2::Oid> {
-                let refname = format!("refs/tags/{}", s);
+    pub fn resolve(
+        &self,
+        repo: &git2::Repository,
+        remote_and_config: Option<(&Url, &Config)>,
+    ) -> CargoResult<git2::Oid> {
+        let id = match self {
+            // Note that we resolve the named tag here in sync with where it's
+            // fetched into via `fetch` below.
+            GitReference::Tag(s) => (|| -> CargoResult<git2::Oid> {
+                let refname = format!("refs/remotes/origin/tags/{}", s);
                 let id = repo.refname_to_id(&refname)?;
                 let obj = repo.find_object(id, None)?;
                 let obj = obj.peel(ObjectType::Commit)?;
                 Ok(obj.id())
             })()
             .chain_err(|| format!("failed to find tag `{}`", s))?,
-            GitReference::Branch(ref s) => {
+
+            // Resolve the remote name since that's all we're configuring in
+            // `fetch` below.
+            GitReference::Branch(s) => {
+                let name = format!("origin/{}", s);
                 let b = repo
-                    .find_branch(s, git2::BranchType::Local)
+                    .find_branch(&name, git2::BranchType::Remote)
                     .chain_err(|| format!("failed to find branch `{}`", s))?;
                 b.get()
                     .target()
                     .ok_or_else(|| anyhow::format_err!("branch `{}` did not have a target", s))?
             }
-            GitReference::Rev(ref s) => {
+
+            // See the module docs for why we're using `master` here.
+            GitReference::DefaultBranch => {
+                let master = repo
+                    .find_branch("origin/master", git2::BranchType::Remote)
+                    .chain_err(|| "failed to find branch `master`")?;
+                let master = master
+                    .get()
+                    .target()
+                    .ok_or_else(|| anyhow::format_err!("branch `master` did not have a target"))?;
+
+                if let Some((remote, config)) = remote_and_config {
+                    let head_id = repo.refname_to_id("refs/remotes/origin/HEAD")?;
+                    let head = repo.find_object(head_id, None)?;
+                    let head = head.peel(ObjectType::Commit)?.id();
+
+                    if head != master {
+                        config.shell().warn(&format!(
+                            "\
+                                fetching `master` branch from `{}` but the `HEAD` \
+                                reference for this repository is not the \
+                                `master` branch. This behavior will change \
+                                in Cargo in the future and your build may \
+                                break, so it's recommended to place \
+                                `branch = \"master\"` in Cargo.toml when \
+                                depending on this git repository to ensure \
+                                that your build will continue to work.\
+                            ",
+                            remote,
+                        ))?;
+                    }
+                }
+                master
+            }
+
+            GitReference::Rev(s) => {
                 let obj = repo.revparse_single(s)?;
                 match obj.as_tag() {
                     Some(tag) => tag.target_id(),
@@ -228,7 +376,7 @@ impl GitReference {
                 }
             }
         };
-        Ok(GitRevision(id))
+        Ok(id)
     }
 }
 
@@ -236,7 +384,7 @@ impl<'a> GitCheckout<'a> {
     fn new(
         path: &Path,
         database: &'a GitDatabase,
-        revision: GitRevision,
+        revision: git2::Oid,
         repo: git2::Repository,
     ) -> GitCheckout<'a> {
         GitCheckout {
@@ -250,7 +398,7 @@ impl<'a> GitCheckout<'a> {
     fn clone_into(
         into: &Path,
         database: &'a GitDatabase,
-        revision: GitRevision,
+        revision: git2::Oid,
         config: &Config,
     ) -> CargoResult<GitCheckout<'a>> {
         let dirname = into.parent().unwrap();
@@ -282,7 +430,6 @@ impl<'a> GitCheckout<'a> {
                 .clone_local(git2::build::CloneLocal::Local)
                 .with_checkout(checkout)
                 .fetch_options(fopts)
-                // .remote_create(|repo, _name, url| repo.remote_anonymous(url))
                 .clone(url.as_str(), into)?;
             repo = Some(r);
             Ok(())
@@ -296,7 +443,7 @@ impl<'a> GitCheckout<'a> {
 
     fn is_fresh(&self) -> bool {
         match self.repo.revparse_single("HEAD") {
-            Ok(ref head) if head.id() == self.revision.0 => {
+            Ok(ref head) if head.id() == self.revision => {
                 // See comments in reset() for why we check this
                 self.location.join(".cargo-ok").exists()
             }
@@ -307,15 +454,11 @@ impl<'a> GitCheckout<'a> {
     fn fetch(&mut self, cargo_config: &Config) -> CargoResult<()> {
         info!("fetch {}", self.repo.path().display());
         let url = self.database.path.into_url()?;
-        let refspec = "refs/heads/*:refs/heads/*";
-        fetch(&mut self.repo, url.as_str(), refspec, cargo_config)?;
+        let reference = GitReference::Rev(self.revision.to_string());
+        fetch(&mut self.repo, url.as_str(), &reference, cargo_config)?;
         Ok(())
     }
 
-    fn has_object(&self) -> bool {
-        self.repo.find_object(self.revision.0, None).is_ok()
-    }
-
     fn reset(&self, config: &Config) -> CargoResult<()> {
         // If we're interrupted while performing this reset (e.g., we die because
         // of a signal) Cargo needs to be sure to try to check out this repo
@@ -328,9 +471,15 @@ impl<'a> GitCheckout<'a> {
         let ok_file = self.location.join(".cargo-ok");
         let _ = paths::remove_file(&ok_file);
         info!("reset {} to {}", self.repo.path().display(), self.revision);
-        let object = self.repo.find_object(self.revision.0, None)?;
+
+        // Ensure libgit2 won't mess with newlines when we vendor.
+        if let Ok(mut git_config) = self.repo.config() {
+            git_config.set_bool("core.autocrlf", false)?;
+        }
+
+        let object = self.repo.find_object(self.revision, None)?;
         reset(&self.repo, &object, config)?;
-        File::create(ok_file)?;
+        paths::create(ok_file)?;
         Ok(())
     }
 
@@ -390,11 +539,11 @@ impl<'a> GitCheckout<'a> {
                 }
             };
             // Fetch data from origin and reset to the head commit
-            let refspec = "refs/heads/*:refs/heads/*";
+            let reference = GitReference::Rev(head.to_string());
             cargo_config
                 .shell()
                 .status("Updating", format!("git submodule `{}`", url))?;
-            fetch(&mut repo, url, refspec, cargo_config).chain_err(|| {
+            fetch(&mut repo, url, &reference, cargo_config).chain_err(|| {
                 format!(
                     "failed to fetch submodule `{}` from {}",
                     child.name().unwrap_or(""),
@@ -448,9 +597,14 @@ where
     let mut ssh_agent_attempts = Vec::new();
     let mut any_attempts = false;
     let mut tried_sshkey = false;
+    let mut url_attempt = None;
 
+    let orig_url = url;
     let mut res = f(&mut |url, username, allowed| {
         any_attempts = true;
+        if url != orig_url {
+            url_attempt = Some(url.to_string());
+        }
         // libgit2's "USERNAME" authentication actually means that it's just
         // asking us for a username to keep going. This is currently only really
         // used for SSH authentication and isn't really an authentication type.
@@ -582,18 +736,26 @@ where
             }
         }
     }
-
-    if res.is_ok() || !any_attempts {
-        return res.map_err(From::from);
-    }
+    let mut err = match res {
+        Ok(e) => return Ok(e),
+        Err(e) => e,
+    };
 
     // In the case of an authentication failure (where we tried something) then
     // we try to give a more helpful error message about precisely what we
     // tried.
-    let res = res.map_err(anyhow::Error::from).chain_err(|| {
+    if any_attempts {
         let mut msg = "failed to authenticate when downloading \
                        repository"
             .to_string();
+
+        if let Some(attempt) = &url_attempt {
+            if url != attempt {
+                msg.push_str(": ");
+                msg.push_str(attempt);
+            }
+        }
+        msg.push_str("\n");
         if !ssh_agent_attempts.is_empty() {
             let names = ssh_agent_attempts
                 .iter()
@@ -601,28 +763,56 @@ where
                 .collect::<Vec<_>>()
                 .join(", ");
             msg.push_str(&format!(
-                "\nattempted ssh-agent authentication, but \
-                 none of the usernames {} succeeded",
+                "\n* attempted ssh-agent authentication, but \
+                 no usernames succeeded: {}",
                 names
             ));
         }
         if let Some(failed_cred_helper) = cred_helper_bad {
             if failed_cred_helper {
                 msg.push_str(
-                    "\nattempted to find username/password via \
+                    "\n* attempted to find username/password via \
                      git's `credential.helper` support, but failed",
                 );
             } else {
                 msg.push_str(
-                    "\nattempted to find username/password via \
+                    "\n* attempted to find username/password via \
                      `credential.helper`, but maybe the found \
                      credentials were incorrect",
                 );
             }
         }
-        msg
-    })?;
-    Ok(res)
+        msg.push_str("\n\n");
+        msg.push_str("if the git CLI succeeds then `net.git-fetch-with-cli` may help here\n");
+        msg.push_str("https://doc.rust-lang.org/cargo/reference/config.html#netgit-fetch-with-cli");
+        err = err.context(msg);
+
+    // Otherwise if we didn't even get to the authentication phase them we may
+    // have failed to set up a connection, in these cases hint on the
+    // `net.git-fetch-with-cli` configuration option.
+    } else if let Some(e) = err.downcast_ref::<git2::Error>() {
+        match e.class() {
+            ErrorClass::Net
+            | ErrorClass::Ssl
+            | ErrorClass::Submodule
+            | ErrorClass::FetchHead
+            | ErrorClass::Ssh
+            | ErrorClass::Callback
+            | ErrorClass::Http => {
+                let mut msg = "network failure seems to have happened\n".to_string();
+                msg.push_str(
+                    "if a proxy or similar is necessary `net.git-fetch-with-cli` may help here\n",
+                );
+                msg.push_str(
+                    "https://doc.rust-lang.org/cargo/reference/config.html#netgit-fetch-with-cli",
+                );
+                err = err.context(msg);
+            }
+            _ => {}
+        }
+    }
+
+    Err(err)
 }
 
 fn reset(repo: &git2::Repository, obj: &git2::Object<'_>, config: &Config) -> CargoResult<()> {
@@ -631,7 +821,9 @@ fn reset(repo: &git2::Repository, obj: &git2::Object<'_>, config: &Config) -> Ca
     opts.progress(|_, cur, max| {
         drop(pb.tick(cur, max));
     });
+    debug!("doing reset");
     repo.reset(obj, git2::ResetType::Hard, Some(&mut opts))?;
+    debug!("reset done");
     Ok(())
 }
 
@@ -656,8 +848,7 @@ pub fn with_fetch_options(
             // Create a local anonymous remote in the repository to fetch the
             // url
             let mut opts = git2::FetchOptions::new();
-            opts.remote_callbacks(rcb)
-                .download_tags(git2::AutotagOption::All);
+            opts.remote_callbacks(rcb);
             cb(opts)
         })?;
         Ok(())
@@ -667,7 +858,7 @@ pub fn with_fetch_options(
 pub fn fetch(
     repo: &mut git2::Repository,
     url: &str,
-    refspec: &str,
+    reference: &GitReference,
     config: &Config,
 ) -> CargoResult<()> {
     if config.frozen() {
@@ -682,19 +873,10 @@ pub fn fetch(
 
     // If we're fetching from GitHub, attempt GitHub's special fast path for
     // testing if we've already got an up-to-date copy of the repository
-
-    if let Ok(url) = Url::parse(url) {
-        if url.host_str() == Some("github.com") {
-            if let Ok(oid) = repo.refname_to_id("refs/remotes/origin/master") {
-                let mut handle = config.http()?.borrow_mut();
-                debug!("attempting GitHub fast path for {}", url);
-                if github_up_to_date(&mut handle, &url, &oid) {
-                    return Ok(());
-                } else {
-                    debug!("fast path failed, falling back to a git fetch");
-                }
-            }
-        }
+    match github_up_to_date(repo, url, reference, config) {
+        Ok(true) => return Ok(()),
+        Ok(false) => {}
+        Err(e) => debug!("failed to check github {:?}", e),
     }
 
     // We reuse repositories quite a lot, so before we go through and update the
@@ -703,6 +885,36 @@ pub fn fetch(
     // request we're about to issue.
     maybe_gc_repo(repo)?;
 
+    // Translate the reference desired here into an actual list of refspecs
+    // which need to get fetched. Additionally record if we're fetching tags.
+    let mut refspecs = Vec::new();
+    let mut tags = false;
+    match reference {
+        // For branches and tags we can fetch simply one reference and copy it
+        // locally, no need to fetch other branches/tags.
+        GitReference::Branch(b) => {
+            refspecs.push(format!("refs/heads/{0}:refs/remotes/origin/{0}", b));
+        }
+        GitReference::Tag(t) => {
+            refspecs.push(format!("refs/tags/{0}:refs/remotes/origin/tags/{0}", t));
+        }
+
+        GitReference::DefaultBranch => {
+            // See the module docs for why we're fetching `master` here.
+            refspecs.push(String::from("refs/heads/master:refs/remotes/origin/master"));
+            refspecs.push(String::from("HEAD:refs/remotes/origin/HEAD"));
+        }
+
+        // For `rev` dependencies we don't know what the rev will point to. To
+        // handle this situation we fetch all branches and tags, and then we
+        // pray it's somewhere in there.
+        GitReference::Rev(_) => {
+            refspecs.push(String::from("refs/heads/*:refs/remotes/origin/*"));
+            refspecs.push(String::from("HEAD:refs/remotes/origin/HEAD"));
+            tags = true;
+        }
+    }
+
     // Unfortunately `libgit2` is notably lacking in the realm of authentication
     // when compared to the `git` command line. As a result, allow an escape
     // hatch for users that would prefer to use `git`-the-CLI for fetching
@@ -710,12 +922,15 @@ pub fn fetch(
     // flavors of authentication possible while also still giving us all the
     // speed and portability of using `libgit2`.
     if let Some(true) = config.net_config()?.git_fetch_with_cli {
-        return fetch_with_cli(repo, url, refspec, config);
+        return fetch_with_cli(repo, url, &refspecs, tags, config);
     }
 
     debug!("doing a fetch for {}", url);
     let git_config = git2::Config::open_default()?;
     with_fetch_options(&git_config, url, config, &mut |mut opts| {
+        if tags {
+            opts.download_tags(git2::AutotagOption::All);
+        }
         // The `fetch` operation here may fail spuriously due to a corrupt
         // repository. It could also fail, however, for a whole slew of other
         // reasons (aka network related reasons). We want Cargo to automatically
@@ -728,10 +943,10 @@ pub fn fetch(
         // blown away the repository, then we want to return the error as-is.
         let mut repo_reinitialized = false;
         loop {
-            debug!("initiating fetch of {} from {}", refspec, url);
+            debug!("initiating fetch of {:?} from {}", refspecs, url);
             let res = repo
                 .remote_anonymous(url)?
-                .fetch(&[refspec], Some(&mut opts), None);
+                .fetch(&refspecs, Some(&mut opts), None);
             let err = match res {
                 Ok(()) => break,
                 Err(e) => e,
@@ -758,16 +973,19 @@ pub fn fetch(
 fn fetch_with_cli(
     repo: &mut git2::Repository,
     url: &str,
-    refspec: &str,
+    refspecs: &[String],
+    tags: bool,
     config: &Config,
 ) -> CargoResult<()> {
     let mut cmd = process("git");
-    cmd.arg("fetch")
-        .arg("--tags") // fetch all tags
-        .arg("--force") // handle force pushes
+    cmd.arg("fetch");
+    if tags {
+        cmd.arg("--tags");
+    }
+    cmd.arg("--force") // handle force pushes
         .arg("--update-head-ok") // see discussion in #2078
         .arg(url)
-        .arg(refspec)
+        .args(refspecs)
         // If cargo is run by git (for example, the `exec` command in `git
         // rebase`), the GIT_DIR is set by git and will point to the wrong
         // location (this takes precedence over the cwd). Make sure this is
@@ -887,9 +1105,9 @@ fn init(path: &Path, bare: bool) -> CargoResult<git2::Repository> {
 /// made.
 ///
 /// This function will attempt to hit that fast path and verify that the `oid`
-/// is actually the current `master` branch of the repository. If `true` is
-/// returned then no update needs to be performed, but if `false` is returned
-/// then the standard update logic still needs to happen.
+/// is actually the current branch of the repository. If `true` is returned then
+/// no update needs to be performed, but if `false` is returned then the
+/// standard update logic still needs to happen.
 ///
 /// [1]: https://developer.github.com/v3/repos/commits/#get-the-sha-1-of-a-commit-reference
 ///
@@ -897,37 +1115,66 @@ fn init(path: &Path, bare: bool) -> CargoResult<git2::Repository> {
 /// just a fast path. As a result all errors are ignored in this function and we
 /// just return a `bool`. Any real errors will be reported through the normal
 /// update path above.
-fn github_up_to_date(handle: &mut Easy, url: &Url, oid: &git2::Oid) -> bool {
-    macro_rules! r#try {
-        ($e:expr) => {
-            match $e {
-                Some(e) => e,
-                None => return false,
-            }
-        };
+fn github_up_to_date(
+    repo: &mut git2::Repository,
+    url: &str,
+    reference: &GitReference,
+    config: &Config,
+) -> CargoResult<bool> {
+    let url = Url::parse(url)?;
+    if url.host_str() != Some("github.com") {
+        return Ok(false);
     }
 
+    let github_branch_name = match reference {
+        GitReference::Branch(branch) => branch,
+        GitReference::Tag(tag) => tag,
+        GitReference::DefaultBranch => "HEAD",
+        GitReference::Rev(_) => {
+            debug!("can't use github fast path with `rev`");
+            return Ok(false);
+        }
+    };
+
     // This expects GitHub urls in the form `github.com/user/repo` and nothing
     // else
-    let mut pieces = r#try!(url.path_segments());
-    let username = r#try!(pieces.next());
-    let repo = r#try!(pieces.next());
+    let mut pieces = url
+        .path_segments()
+        .ok_or_else(|| anyhow!("no path segments on url"))?;
+    let username = pieces
+        .next()
+        .ok_or_else(|| anyhow!("couldn't find username"))?;
+    let repository = pieces
+        .next()
+        .ok_or_else(|| anyhow!("couldn't find repository name"))?;
     if pieces.next().is_some() {
-        return false;
+        anyhow::bail!("too many segments on URL");
     }
 
-    let url = format!(
-        "https://api.github.com/repos/{}/{}/commits/master",
-        username, repo
-    );
-    r#try!(handle.get(true).ok());
-    r#try!(handle.url(&url).ok());
-    r#try!(handle.useragent("cargo").ok());
-    let mut headers = List::new();
-    r#try!(headers.append("Accept: application/vnd.github.3.sha").ok());
-    r#try!(headers.append(&format!("If-None-Match: \"{}\"", oid)).ok());
-    r#try!(handle.http_headers(headers).ok());
-    r#try!(handle.perform().ok());
+    // Trim off the `.git` from the repository, if present, since that's
+    // optional for GitHub and won't work when we try to use the API as well.
+    let repository = if repository.ends_with(".git") {
+        &repository[..repository.len() - 4]
+    } else {
+        repository
+    };
 
-    r#try!(handle.response_code().ok()) == 304
+    let url = format!(
+        "https://api.github.com/repos/{}/{}/commits/{}",
+        username, repository, github_branch_name,
+    );
+    let mut handle = config.http()?.borrow_mut();
+    debug!("attempting GitHub fast path for {}", url);
+    handle.get(true)?;
+    handle.url(&url)?;
+    handle.useragent("cargo")?;
+    let mut headers = List::new();
+    headers.append("Accept: application/vnd.github.3.sha")?;
+    headers.append(&format!(
+        "If-None-Match: \"{}\"",
+        reference.resolve(repo, None)?
+    ))?;
+    handle.http_headers(headers)?;
+    handle.perform()?;
+    Ok(handle.response_code()? == 304)
 }
