@@ -67,8 +67,9 @@
 //! hopefully those are more obvious inline in the code itself.
 
 use crate::core::dependency::Dependency;
-use crate::core::{InternedString, PackageId, SourceId, Summary};
+use crate::core::{PackageId, SourceId, Summary};
 use crate::sources::registry::{RegistryData, RegistryPackage};
+use crate::util::interning::InternedString;
 use crate::util::paths;
 use crate::util::{internal, CargoResult, Config, Filesystem, ToSemver};
 use log::info;
@@ -115,7 +116,7 @@ impl<'s> Iterator for UncanonicalizedIter<'s> {
                 .chars()
                 .scan(0u16, |s, c| {
                     // the check against 15 here's to prevent
-                    // shift overflow on inputs with more then 15 hyphens
+                    // shift overflow on inputs with more than 15 hyphens
                     if (c == '_' || c == '-') && *s <= 15 {
                         let switch = (self.hyphen_combination_num & (1u16 << *s)) > 0;
                         let out = if (c == '_') ^ switch { '_' } else { '-' };
