@@ -4,8 +4,7 @@ use cargo::util::IntoUrl;
 use cargo_test_support::publish::validate_alt_upload;
 use cargo_test_support::registry::{self, Package};
 use cargo_test_support::{basic_manifest, git, paths, project};
-use std::fs::{self, File};
-use std::io::Write;
+use std::fs;
 
 #[cargo_test]
 fn depend_on_alt_registry() {
@@ -299,7 +298,7 @@ fn cannot_publish_to_crates_io_with_registry_dependency() {
         .with_stderr_contains("[ERROR] crates cannot be published to crates.io[..]")
         .run();
 
-    p.cargo("publish --index")
+    p.cargo("publish --token sekrit --index")
         .arg(fakeio_url.to_string())
         .with_status(101)
         .with_stderr_contains("[ERROR] crates cannot be published to crates.io[..]")
@@ -414,17 +413,18 @@ fn alt_registry_and_crates_io_deps() {
 
 #[cargo_test]
 fn block_publish_due_to_no_token() {
-    let p = project().file("src/main.rs", "fn main() {}").build();
-
-    // Setup the registry by publishing a package
-    Package::new("bar", "0.0.1").alternative(true).publish();
+    registry::init();
+    let p = project().file("src/lib.rs", "").build();
 
     fs::remove_file(paths::home().join(".cargo/credentials")).unwrap();
 
     // Now perform the actual publish
     p.cargo("publish --registry alternative")
         .with_status(101)
-        .with_stderr_contains("error: no upload token found, please run `cargo login`")
+        .with_stderr_contains(
+            "error: no upload token found, \
+            please run `cargo login` or pass `--token`",
+        )
         .run();
 }
 
@@ -529,51 +529,33 @@ fn publish_with_crates_io_dep() {
     );
 }
 
-#[cargo_test]
-fn passwords_in_registry_index_url_forbidden() {
-    registry::init();
-
-    let config = paths::home().join(".cargo/config");
-
-    File::create(config)
-        .unwrap()
-        .write_all(
-            br#"
-        [registry]
-        index = "ssh://git:secret@foobar.com"
-        "#,
-        )
-        .unwrap();
-
-    let p = project().file("src/main.rs", "fn main() {}").build();
-
-    p.cargo("publish")
-        .with_status(101)
-        .with_stderr_contains("error: Registry URLs may not contain passwords")
-        .run();
-}
-
 #[cargo_test]
 fn passwords_in_registries_index_url_forbidden() {
     registry::init();
 
     let config = paths::home().join(".cargo/config");
 
-    File::create(config)
-        .unwrap()
-        .write_all(
-            br#"
+    fs::write(
+        config,
+        r#"
         [registries.alternative]
         index = "ssh://git:secret@foobar.com"
         "#,
-        )
-        .unwrap();
+    )
+    .unwrap();
 
     let p = project().file("src/main.rs", "fn main() {}").build();
 
     p.cargo("publish --registry alternative")
         .with_status(101)
-        .with_stderr_contains("error: Registry URLs may not contain passwords")
+        .with_stderr(
+            "\
+error: invalid index URL for registry `alternative` defined in [..]/home/.cargo/config
+
+Caused by:
+  registry URLs may not contain passwords
+",
+        )
         .run();
 }
 
@@ -842,7 +824,8 @@ fn alt_reg_metadata() {
                 "resolve": null,
                 "target_directory": "[..]/foo/target",
                 "version": 1,
-                "workspace_root": "[..]/foo"
+                "workspace_root": "[..]/foo",
+                "metadata": null
             }"#,
         )
         .run();
@@ -853,28 +836,6 @@ fn alt_reg_metadata() {
             r#"
              {
                 "packages": [
-                    {
-                        "name": "altdep2",
-                        "version": "0.0.1",
-                        "id": "altdep2 0.0.1 (registry+file:[..]/alternative-registry)",
-                        "license": null,
-                        "license_file": null,
-                        "description": null,
-                        "source": "registry+file:[..]/alternative-registry",
-                        "dependencies": [],
-                        "targets": "{...}",
-                        "features": {},
-                        "manifest_path": "[..]/altdep2-0.0.1/Cargo.toml",
-                        "metadata": null,
-                        "publish": null,
-                        "authors": [],
-                        "categories": [],
-                        "keywords": [],
-                        "readme": null,
-                        "repository": null,
-                        "edition": "2015",
-                        "links": null
-                    },
                     {
                         "name": "altdep",
                         "version": "0.0.1",
@@ -910,6 +871,50 @@ fn alt_reg_metadata() {
                         "edition": "2015",
                         "links": null
                     },
+                    {
+                        "name": "altdep2",
+                        "version": "0.0.1",
+                        "id": "altdep2 0.0.1 (registry+file:[..]/alternative-registry)",
+                        "license": null,
+                        "license_file": null,
+                        "description": null,
+                        "source": "registry+file:[..]/alternative-registry",
+                        "dependencies": [],
+                        "targets": "{...}",
+                        "features": {},
+                        "manifest_path": "[..]/altdep2-0.0.1/Cargo.toml",
+                        "metadata": null,
+                        "publish": null,
+                        "authors": [],
+                        "categories": [],
+                        "keywords": [],
+                        "readme": null,
+                        "repository": null,
+                        "edition": "2015",
+                        "links": null
+                    },
+                    {
+                        "name": "bar",
+                        "version": "0.0.1",
+                        "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
+                        "license": null,
+                        "license_file": null,
+                        "description": null,
+                        "source": "registry+https://github.com/rust-lang/crates.io-index",
+                        "dependencies": [],
+                        "targets": "{...}",
+                        "features": {},
+                        "manifest_path": "[..]/bar-0.0.1/Cargo.toml",
+                        "metadata": null,
+                        "publish": null,
+                        "authors": [],
+                        "categories": [],
+                        "keywords": [],
+                        "readme": null,
+                        "repository": null,
+                        "edition": "2015",
+                        "links": null
+                    },
                     {
                         "name": "foo",
                         "version": "0.0.1",
@@ -991,28 +996,6 @@ fn alt_reg_metadata() {
                         "repository": null,
                         "edition": "2015",
                         "links": null
-                    },
-                    {
-                        "name": "bar",
-                        "version": "0.0.1",
-                        "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
-                        "license": null,
-                        "license_file": null,
-                        "description": null,
-                        "source": "registry+https://github.com/rust-lang/crates.io-index",
-                        "dependencies": [],
-                        "targets": "{...}",
-                        "features": {},
-                        "manifest_path": "[..]/bar-0.0.1/Cargo.toml",
-                        "metadata": null,
-                        "publish": null,
-                        "authors": [],
-                        "categories": [],
-                        "keywords": [],
-                        "readme": null,
-                        "repository": null,
-                        "edition": "2015",
-                        "links": null
                     }
                 ],
                 "workspace_members": [
@@ -1021,7 +1004,8 @@ fn alt_reg_metadata() {
                 "resolve": "{...}",
                 "target_directory": "[..]/foo/target",
                 "version": 1,
-                "workspace_root": "[..]/foo"
+                "workspace_root": "[..]/foo",
+                "metadata": null
             }"#,
         )
         .run();
@@ -1071,6 +1055,41 @@ fn unknown_registry() {
             r#"
             {
               "packages": [
+                {
+                  "name": "bar",
+                  "version": "0.0.1",
+                  "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
+                  "license": null,
+                  "license_file": null,
+                  "description": null,
+                  "source": "registry+https://github.com/rust-lang/crates.io-index",
+                  "dependencies": [
+                    {
+                      "name": "baz",
+                      "source": "registry+file://[..]/alternative-registry",
+                      "req": "^0.0.1",
+                      "kind": null,
+                      "rename": null,
+                      "optional": false,
+                      "uses_default_features": true,
+                      "features": [],
+                      "target": null,
+                      "registry": "file:[..]/alternative-registry"
+                    }
+                  ],
+                  "targets": "{...}",
+                  "features": {},
+                  "manifest_path": "[..]",
+                  "metadata": null,
+                  "publish": null,
+                  "authors": [],
+                  "categories": [],
+                  "keywords": [],
+                  "readme": null,
+                  "repository": null,
+                  "edition": "2015",
+                  "links": null
+                },
                 {
                   "name": "baz",
                   "version": "0.0.1",
@@ -1127,41 +1146,6 @@ fn unknown_registry() {
                   "repository": null,
                   "edition": "2015",
                   "links": null
-                },
-                {
-                  "name": "bar",
-                  "version": "0.0.1",
-                  "id": "bar 0.0.1 (registry+https://github.com/rust-lang/crates.io-index)",
-                  "license": null,
-                  "license_file": null,
-                  "description": null,
-                  "source": "registry+https://github.com/rust-lang/crates.io-index",
-                  "dependencies": [
-                    {
-                      "name": "baz",
-                      "source": "registry+file://[..]/alternative-registry",
-                      "req": "^0.0.1",
-                      "kind": null,
-                      "rename": null,
-                      "optional": false,
-                      "uses_default_features": true,
-                      "features": [],
-                      "target": null,
-                      "registry": "file:[..]/alternative-registry"
-                    }
-                  ],
-                  "targets": "{...}",
-                  "features": {},
-                  "manifest_path": "[..]",
-                  "metadata": null,
-                  "publish": null,
-                  "authors": [],
-                  "categories": [],
-                  "keywords": [],
-                  "readme": null,
-                  "repository": null,
-                  "edition": "2015",
-                  "links": null
                 }
               ],
               "workspace_members": [
@@ -1170,7 +1154,8 @@ fn unknown_registry() {
               "resolve": "{...}",
               "target_directory": "[..]/foo/target",
               "version": 1,
-              "workspace_root": "[..]/foo"
+              "workspace_root": "[..]/foo",
+              "metadata": null
             }
             "#,
         )
@@ -1181,15 +1166,14 @@ fn unknown_registry() {
 fn registries_index_relative_url() {
     let config = paths::root().join(".cargo/config");
     fs::create_dir_all(config.parent().unwrap()).unwrap();
-    File::create(&config)
-        .unwrap()
-        .write_all(
-            br#"
+    fs::write(
+        &config,
+        r#"
             [registries.relative]
             index = "file:alternative-registry"
         "#,
-        )
-        .unwrap();
+    )
+    .unwrap();
 
     registry::init();
 
@@ -1227,71 +1211,18 @@ fn registries_index_relative_url() {
         .run();
 }
 
-#[cargo_test]
-fn registry_index_relative_url() {
-    let config = paths::root().join(".cargo/config");
-    fs::create_dir_all(config.parent().unwrap()).unwrap();
-    File::create(&config)
-        .unwrap()
-        .write_all(
-            br#"
-            [registry]
-            index = "file:alternative-registry"
-        "#,
-        )
-        .unwrap();
-
-    registry::init();
-
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [dependencies.bar]
-            version = "0.0.1"
-        "#,
-        )
-        .file("src/main.rs", "fn main() {}")
-        .build();
-
-    Package::new("bar", "0.0.1").alternative(true).publish();
-
-    fs::remove_file(paths::home().join(".cargo/config")).unwrap();
-
-    p.cargo("build")
-        .with_stderr(&format!(
-            "\
-warning: custom registry support via the `registry.index` configuration is being removed, this functionality will not work in the future
-[UPDATING] `{reg}` index
-[DOWNLOADING] crates ...
-[DOWNLOADED] bar v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] bar v0.0.1 (registry `[ROOT][..]`)
-[COMPILING] foo v0.0.1 ([CWD])
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]s
-",
-            reg = registry::alt_registry_path().to_str().unwrap()
-        ))
-        .run();
-}
-
 #[cargo_test]
 fn registries_index_relative_path_not_allowed() {
     let config = paths::root().join(".cargo/config");
     fs::create_dir_all(config.parent().unwrap()).unwrap();
-    File::create(&config)
-        .unwrap()
-        .write_all(
-            br#"
+    fs::write(
+        &config,
+        r#"
             [registries.relative]
             index = "alternative-registry"
         "#,
-        )
-        .unwrap();
+    )
+    .unwrap();
 
     registry::init();
 
@@ -1319,6 +1250,9 @@ fn registries_index_relative_path_not_allowed() {
             "\
 error: failed to parse manifest at `{root}/foo/Cargo.toml`
 
+Caused by:
+  invalid index URL for registry `relative` defined in [..]/.cargo/config
+
 Caused by:
   invalid url `alternative-registry`: relative URL without a base
 ",
@@ -1327,3 +1261,19 @@ Caused by:
         .with_status(101)
         .run();
 }
+
+#[cargo_test]
+fn both_index_and_registry() {
+    let p = project().file("src/lib.rs", "").build();
+    for cmd in &["publish", "owner", "search", "yank --vers 1.0.0"] {
+        p.cargo(cmd)
+            .arg("--registry=foo")
+            .arg("--index=foo")
+            .with_status(101)
+            .with_stderr(
+                "[ERROR] both `--index` and `--registry` \
+                should not be set at the same time",
+            )
+            .run();
+    }
+}
