@@ -1,3 +1,5 @@
+//! Tests for the `cargo bench` command.
+
 use cargo_test_support::is_nightly;
 use cargo_test_support::paths::CargoPathExt;
 use cargo_test_support::{basic_bin_manifest, basic_lib_manifest, basic_manifest, project};
@@ -594,6 +596,9 @@ fn bench_autodiscover_2015() {
                 authors = []
                 edition = "2015"
 
+                [features]
+                magic = []
+
                 [[bench]]
                 name = "bench_magic"
                 required-features = ["magic"]
@@ -1610,9 +1615,9 @@ fn json_artifact_includes_executable_for_benchmark() {
         .with_json(
             r#"
             {
-                "executable": "[..]/foo/target/release/benchmark-[..][EXE]",
+                "executable": "[..]/foo/target/release/deps/benchmark-[..][EXE]",
                 "features": [],
-                "filenames": [ "[..]/foo/target/release/benchmark-[..][EXE]" ],
+                "filenames": "{...}",
                 "fresh": false,
                 "package_id": "foo 0.0.1 ([..])",
                 "profile": "{...}",
@@ -1623,9 +1628,12 @@ fn json_artifact_includes_executable_for_benchmark() {
                     "doctest": false,
                     "edition": "2015",
                     "name": "benchmark",
-                    "src_path": "[..]/foo/benches/benchmark.rs"
+                    "src_path": "[..]/foo/benches/benchmark.rs",
+                    "test": false
                 }
             }
+
+            {"reason": "build-finished", "success": true}
         "#,
         )
         .run();
