@@ -1,9 +1,8 @@
-use std::fs::File;
-use std::io::prelude::*;
+//! Tests for `[features]` table.
 
-use crate::support::paths::CargoPathExt;
-use crate::support::registry::Package;
-use crate::support::{basic_manifest, project};
+use cargo_test_support::paths::CargoPathExt;
+use cargo_test_support::registry::{Dependency, Package};
+use cargo_test_support::{basic_manifest, project};
 
 #[cargo_test]
 fn invalid1() {
@@ -99,7 +98,7 @@ fn invalid3() {
 
 Caused by:
   Feature `bar` depends on `baz` which is not an optional dependency.
-Consider adding `optional = true` to the dependency
+  Consider adding `optional = true` to the dependency
 ",
         )
         .run();
@@ -292,7 +291,7 @@ fn invalid9() {
         .build();
 
     p.cargo("build --features bar")
-.with_stderr(
+        .with_stderr(
             "\
 error: Package `foo v0.0.1 ([..])` does not have feature `bar`. It has a required dependency with that name, but only optional dependencies can be used as features.
 ",
@@ -918,9 +917,7 @@ fn everything_in_the_lockfile() {
         .build();
 
     p.cargo("fetch").run();
-    let loc = p.root().join("Cargo.lock");
-    let mut lockfile = String::new();
-    t!(t!(File::open(&loc)).read_to_string(&mut lockfile));
+    let lockfile = p.read_lockfile();
     assert!(
         lockfile.contains(r#"name = "d1""#),
         "d1 not found\n{}",
@@ -1356,80 +1353,6 @@ fn many_cli_features_comma_and_space_delimited() {
         .run();
 }
 
-#[cargo_test]
-fn combining_features_and_package() {
-    Package::new("dep", "1.0.0").publish();
-
-    let p = project()
-        .file(
-            "Cargo.toml",
-            r#"
-            [project]
-            name = "foo"
-            version = "0.0.1"
-            authors = []
-
-            [workspace]
-            members = ["bar"]
-
-            [dependencies]
-            dep = "1"
-        "#,
-        )
-        .file("src/lib.rs", "")
-        .file(
-            "bar/Cargo.toml",
-            r#"
-            [package]
-            name = "bar"
-            version = "0.0.1"
-            authors = []
-            [features]
-            main = []
-        "#,
-        )
-        .file(
-            "bar/src/main.rs",
-            r#"
-            #[cfg(feature = "main")]
-            fn main() {}
-        "#,
-        )
-        .build();
-
-    p.cargo("build -Z package-features --all --features main")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for more than one package")
-        .run();
-
-    p.cargo("build -Z package-features --package dep --features main")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-    p.cargo("build -Z package-features --package dep --all-features")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-    p.cargo("build -Z package-features --package dep --no-default-features")
-        .masquerade_as_nightly_cargo()
-        .with_status(101)
-        .with_stderr_contains("[ERROR] cannot specify features for packages outside of workspace")
-        .run();
-
-    p.cargo("build -Z package-features --all --all-features")
-        .masquerade_as_nightly_cargo()
-        .run();
-    p.cargo("run -Z package-features --package bar --features main")
-        .masquerade_as_nightly_cargo()
-        .run();
-    p.cargo("build -Z package-features --package dep")
-        .masquerade_as_nightly_cargo()
-        .run();
-}
-
 #[cargo_test]
 fn namespaced_invalid_feature() {
     let p = project()
@@ -1533,7 +1456,7 @@ fn namespaced_non_optional_dependency() {
 
 Caused by:
   Feature `bar` includes `crate:baz` which is not an optional dependency.
-Consider adding `optional = true` to the dependency
+  Consider adding `optional = true` to the dependency
 ",
         )
         .run();
@@ -1591,14 +1514,14 @@ fn namespaced_shadowed_dep() {
         .build();
 
     p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
-            "\
+        "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `baz` includes the optional dependency of the same name, but this is left implicit in the features included by this feature.
-Consider adding `crate:baz` to this feature's requirements.
+  Consider adding `crate:baz` to this feature's requirements.
 ",
-        )
+    )
         .run();
 }
 
@@ -1627,15 +1550,15 @@ fn namespaced_shadowed_non_optional() {
         .build();
 
     p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
-            "\
+        "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `baz` includes the dependency of the same name, but this is left implicit in the features included by this feature.
-Additionally, the dependency must be marked as optional to be included in the feature definition.
-Consider adding `crate:baz` to this feature's requirements and marking the dependency as `optional = true`
+  Additionally, the dependency must be marked as optional to be included in the feature definition.
+  Consider adding `crate:baz` to this feature's requirements and marking the dependency as `optional = true`
 ",
-        )
+    )
         .run();
 }
 
@@ -1664,15 +1587,14 @@ fn namespaced_implicit_non_optional() {
         .build();
 
     p.cargo("build").masquerade_as_nightly_cargo().with_status(101).with_stderr(
-            "\
+        "\
 [ERROR] failed to parse manifest at `[..]`
 
 Caused by:
   Feature `bar` includes `baz` which is not defined as a feature.
-A non-optional dependency of the same name is defined; consider adding `optional = true` to its definition
+  A non-optional dependency of the same name is defined; consider adding `optional = true` to its definition
 ",
-        ).run(
-    );
+    ).run();
 }
 
 #[cargo_test]
@@ -1764,7 +1686,7 @@ fn all_features_all_crates() {
         .file("bar/src/main.rs", "#[cfg(feature = \"foo\")] fn main() {}")
         .build();
 
-    p.cargo("build --all-features --all").run();
+    p.cargo("build --all-features --workspace").run();
 }
 
 #[cargo_test]
@@ -1959,3 +1881,355 @@ fn multi_multi_features() {
 
     p.cargo("build --features a --features").arg("b c").run();
 }
+
+#[cargo_test]
+fn cli_parse_ok() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [project]
+                name = "foo"
+                version = "0.0.1"
+                authors = []
+
+                [features]
+                a = []
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+               #[cfg(feature = "a")]
+               fn main() {
+                    assert_eq!(std::env::args().nth(1).unwrap(), "b");
+               }
+            "#,
+        )
+        .build();
+
+    p.cargo("run --features a b").run();
+}
+
+#[cargo_test]
+fn virtual_ws_flags() {
+    // Reject features flags in the root of a virtual workspace.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [workspace]
+                members = ["a"]
+            "#,
+        )
+        .file(
+            "a/Cargo.toml",
+            r#"
+                [package]
+                name = "a"
+                version = "0.1.0"
+
+                [features]
+                f1 = []
+            "#,
+        )
+        .file("a/src/lib.rs", "")
+        .build();
+
+    p.cargo("build --features=f1")
+        .with_stderr(
+            "[ERROR] --features is not allowed in the root of a virtual workspace\n\
+             note: while this was previously accepted, it didn't actually do anything",
+        )
+        .with_status(101)
+        .run();
+
+    p.cargo("build --no-default-features")
+        .with_stderr(
+            "[ERROR] --no-default-features is not allowed in the root of a virtual workspace\n\
+             note: while this was previously accepted, it didn't actually do anything",
+        )
+        .with_status(101)
+        .run();
+
+    // It's OK if cwd is in a member.
+    p.cargo("check --features=f1 -v")
+        .cwd("a")
+        .with_stderr(
+            "\
+[CHECKING] a [..]
+[RUNNING] `rustc --crate-name a a/src/lib.rs [..]--cfg [..]feature[..]f1[..]
+[FINISHED] dev [..]
+",
+        )
+        .run();
+
+    p.cargo("clean").run();
+
+    // And -Zpackage-features is OK because it is designed to support this.
+    p.cargo("check --features=f1 -p a -Z package-features -v")
+        .masquerade_as_nightly_cargo()
+        .with_stderr(
+            "\
+[CHECKING] a [..]
+[RUNNING] `rustc --crate-name a a/src/lib.rs [..]--cfg [..]feature[..]f1[..]
+[FINISHED] dev [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn all_features_virtual_ws() {
+    // What happens with `--all-features` in the root of a virtual workspace.
+    // Some of this behavior is a little strange (member dependencies also
+    // have all features enabled, one might expect `f4` to be disabled).
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [workspace]
+                members = ["a", "b"]
+            "#,
+        )
+        .file(
+            "a/Cargo.toml",
+            r#"
+                [package]
+                name = "a"
+                version = "0.1.0"
+                edition = "2018"
+
+                [dependencies]
+                b = {path="../b", optional=true}
+
+                [features]
+                default = ["f1"]
+                f1 = []
+                f2 = []
+            "#,
+        )
+        .file(
+            "a/src/main.rs",
+            r#"
+                fn main() {
+                    if cfg!(feature="f1") {
+                        println!("f1");
+                    }
+                    if cfg!(feature="f2") {
+                        println!("f2");
+                    }
+                    #[cfg(feature="b")]
+                    b::f();
+                }
+            "#,
+        )
+        .file(
+            "b/Cargo.toml",
+            r#"
+                [package]
+                name = "b"
+                version = "0.1.0"
+
+                [features]
+                default = ["f3"]
+                f3 = []
+                f4 = []
+            "#,
+        )
+        .file(
+            "b/src/lib.rs",
+            r#"
+                pub fn f() {
+                    if cfg!(feature="f3") {
+                        println!("f3");
+                    }
+                    if cfg!(feature="f4") {
+                        println!("f4");
+                    }
+                }
+            "#,
+        )
+        .build();
+
+    p.cargo("run").with_stdout("f1\n").run();
+    p.cargo("run --all-features")
+        .with_stdout("f1\nf2\nf3\nf4\n")
+        .run();
+    // In `a`, it behaves differently. :(
+    p.cargo("run --all-features")
+        .cwd("a")
+        .with_stdout("f1\nf2\nf3\n")
+        .run();
+}
+
+#[cargo_test]
+fn slash_optional_enables() {
+    // --features dep/feat will enable `dep` and set its feature.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+
+            [dependencies]
+            dep = {path="dep", optional=true}
+            "#,
+        )
+        .file(
+            "src/lib.rs",
+            r#"
+            #[cfg(not(feature="dep"))]
+            compile_error!("dep not set");
+            "#,
+        )
+        .file(
+            "dep/Cargo.toml",
+            r#"
+            [package]
+            name = "dep"
+            version = "0.1.0"
+
+            [features]
+            feat = []
+            "#,
+        )
+        .file(
+            "dep/src/lib.rs",
+            r#"
+            #[cfg(not(feature="feat"))]
+            compile_error!("feat not set");
+            "#,
+        )
+        .build();
+
+    p.cargo("check")
+        .with_status(101)
+        .with_stderr_contains("[..]dep not set[..]")
+        .run();
+
+    p.cargo("check --features dep/feat").run();
+}
+
+#[cargo_test]
+fn registry_summary_order_doesnt_matter() {
+    // Checks for an issue where the resolver depended on the order of entries
+    // in the registry summary. If there was a non-optional dev-dependency
+    // that appeared before an optional normal dependency, then the resolver
+    // would not activate the optional dependency with a pkg/featname feature
+    // syntax.
+    Package::new("dep", "0.1.0")
+        .feature("feat1", &[])
+        .file(
+            "src/lib.rs",
+            r#"
+                #[cfg(feature="feat1")]
+                pub fn work() {
+                    println!("it works");
+                }
+            "#,
+        )
+        .publish();
+    Package::new("bar", "0.1.0")
+        .feature("bar_feat", &["dep/feat1"])
+        .add_dep(Dependency::new("dep", "0.1.0").dev())
+        .add_dep(Dependency::new("dep", "0.1.0").optional(true))
+        .file(
+            "src/lib.rs",
+            r#"
+                // This will fail to compile without `dep` optional dep activated.
+                extern crate dep;
+
+                pub fn doit() {
+                    dep::work();
+                }
+            "#,
+        )
+        .publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                edition = "2018"
+
+                [dependencies]
+                bar = { version="0.1", features = ["bar_feat"] }
+            "#,
+        )
+        .file(
+            "src/main.rs",
+            r#"
+                fn main() {
+                    bar::doit();
+                }
+            "#,
+        )
+        .build();
+
+    p.cargo("run")
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[DOWNLOADING] crates ...
+[DOWNLOADED] [..]
+[DOWNLOADED] [..]
+[COMPILING] dep v0.1.0
+[COMPILING] bar v0.1.0
+[COMPILING] foo v0.1.0 [..]
+[FINISHED] [..]
+[RUNNING] `target/debug/foo[EXE]`
+",
+        )
+        .with_stdout("it works")
+        .run();
+}
+
+#[cargo_test]
+fn nonexistent_required_features() {
+    Package::new("required_dependency", "0.1.0")
+        .feature("simple", &[])
+        .publish();
+    Package::new("optional_dependency", "0.2.0")
+        .feature("optional", &[])
+        .publish();
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.1.0"
+            [features]
+            existing = []
+            fancy = ["optional_dependency"]
+            [dependencies]
+            required_dependency = { version = "0.1", optional = false}
+            optional_dependency = { version = "0.2", optional = true}
+            [[example]]
+            name = "ololo"
+            required-features = ["not_present",
+                                 "existing",
+                                 "fancy",
+                                 "required_dependency/not_existing",
+                                 "required_dependency/simple",
+                                 "optional_dependency/optional",
+                                 "not_specified_dependency/some_feature"]"#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .file("examples/ololo.rs", "fn main() {}")
+        .build();
+
+    p.cargo("build --examples")
+        .with_stderr_contains(
+            r#"warning: feature `not_present` is not present in [features] section.
+warning: feature `not_existing` does not exist in package `required_dependency v0.1.0`.
+warning: dependency `not_specified_dependency` specified in required-features as `not_specified_dependency/some_feature` does not exist."#,
+        )
+        .run();
+}
