@@ -1,11 +1,8 @@
 //! Tests for the `cargo fix` command.
 
-use std::fs::File;
-
 use cargo_test_support::git;
-use cargo_test_support::{basic_manifest, command_is_available, project};
-
-use std::io::Write;
+use cargo_test_support::paths;
+use cargo_test_support::{basic_manifest, project};
 
 #[cargo_test]
 fn do_not_fix_broken_builds() {
@@ -689,7 +686,7 @@ fn shows_warnings() {
         .build();
 
     p.cargo("fix --allow-no-vcs")
-        .with_stderr_contains("[..]warning: use of deprecated item[..]")
+        .with_stderr_contains("[..]warning: use of deprecated[..]")
         .run();
 }
 
@@ -712,7 +709,7 @@ fn warns_if_no_vcs_detected() {
 fn warns_about_dirty_working_directory() {
     let p = git::new("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    File::create(p.root().join("src/lib.rs")).unwrap();
+    p.change_file("src/lib.rs", "");
 
     p.cargo("fix")
         .with_status(101)
@@ -736,10 +733,7 @@ commit the changes to these files:
 fn warns_about_staged_working_directory() {
     let (p, repo) = git::new_repo("foo", |p| p.file("src/lib.rs", "pub fn foo() {}"));
 
-    File::create(&p.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all("pub fn bar() {}".to_string().as_bytes())
-        .unwrap();
+    p.change_file("src/lib.rs", "pub fn bar() {}");
     git::add(&repo);
 
     p.cargo("fix")
@@ -773,7 +767,7 @@ fn does_not_warn_about_dirty_ignored_files() {
             .file(".gitignore", "bar\n")
     });
 
-    File::create(p.root().join("bar")).unwrap();
+    p.change_file("bar", "");
 
     p.cargo("fix").run();
 }
@@ -935,11 +929,11 @@ fn shows_warnings_on_second_run_without_changes() {
         .build();
 
     p.cargo("fix --allow-no-vcs")
-        .with_stderr_contains("[..]warning: use of deprecated item[..]")
+        .with_stderr_contains("[..]warning: use of deprecated[..]")
         .run();
 
     p.cargo("fix --allow-no-vcs")
-        .with_stderr_contains("[..]warning: use of deprecated item[..]")
+        .with_stderr_contains("[..]warning: use of deprecated[..]")
         .run();
 }
 
@@ -1068,11 +1062,8 @@ fn doesnt_rebuild_dependencies() {
 }
 
 #[cargo_test]
+#[cfg(unix)]
 fn does_not_crash_with_rustc_wrapper() {
-    // We don't have /usr/bin/env on Windows.
-    if cfg!(windows) {
-        return;
-    }
     let p = project()
         .file(
             "Cargo.toml",
@@ -1090,6 +1081,49 @@ fn does_not_crash_with_rustc_wrapper() {
         .run();
 }
 
+#[cargo_test]
+#[cfg(unix)]
+fn does_not_crash_with_rustc_workspace_wrapper() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("fix --allow-no-vcs --verbose -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", "/usr/bin/env")
+        .masquerade_as_nightly_cargo()
+        .run();
+}
+
+#[cargo_test]
+fn uses_workspace_wrapper_and_primary_wrapper_override() {
+    // We don't have /usr/bin/env on Windows.
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("fix --allow-no-vcs --verbose -Zunstable-options")
+        .env("RUSTC_WORKSPACE_WRAPPER", paths::echo_wrapper())
+        .masquerade_as_nightly_cargo()
+        .with_stderr_contains("WRAPPER CALLED: rustc src/lib.rs --crate-name foo [..]")
+        .run();
+}
+
 #[cargo_test]
 fn only_warn_for_relevant_crates() {
     let p = project()
@@ -1251,47 +1285,6 @@ fn fix_in_existing_repo_weird_ignore() {
     p.cargo("fix").cwd("src").run();
 }
 
-#[cargo_test]
-fn fix_with_clippy() {
-    if !command_is_available("clippy-driver") {
-        return;
-    }
-
-    let p = project()
-        .file(
-            "src/lib.rs",
-            "
-                pub fn foo() {
-                    let mut v = Vec::<String>::new();
-                    let _ = v.iter_mut().filter(|&ref a| a.is_empty());
-                }
-    ",
-        )
-        .build();
-
-    let stderr = "\
-[CHECKING] foo v0.0.1 ([..])
-[FIXING] src/lib.rs (1 fix)
-[FINISHED] [..]
-";
-
-    p.cargo("fix -Zunstable-options --clippy --allow-no-vcs")
-        .masquerade_as_nightly_cargo()
-        .with_stderr(stderr)
-        .with_stdout("")
-        .run();
-
-    assert_eq!(
-        p.read_file("src/lib.rs"),
-        "
-                pub fn foo() {
-                    let mut v = Vec::<String>::new();
-                    let _ = v.iter_mut().filter(|a| a.is_empty());
-                }
-    "
-    );
-}
-
 #[cargo_test]
 fn fix_color_message() {
     // Check that color appears in diagnostics.
