@@ -1,10 +1,11 @@
 //! Tests for git support.
 
 use std::env;
-use std::fs::{self, File};
+use std::fs;
 use std::io::prelude::*;
 use std::net::{TcpListener, TcpStream};
 use std::path::Path;
+use std::str;
 use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::Arc;
 use std::thread;
@@ -501,10 +502,7 @@ fn two_revs_same_deps() {
     let rev1 = repo.revparse_single("HEAD").unwrap().id();
 
     // Commit the changes and make sure we trigger a recompile
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     git::add(&repo);
     let rev2 = git::commit(&repo);
 
@@ -623,10 +621,7 @@ fn recompilation() {
     p.cargo("build").with_stdout("").run();
 
     // Modify a file manually, shouldn't trigger a recompile
-    File::create(&git_project.root().join("src/bar.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() { println!("hello!"); }"#)
-        .unwrap();
+    git_project.change_file("src/bar.rs", r#"pub fn bar() { println!("hello!"); }"#);
 
     p.cargo("build").with_stdout("").run();
 
@@ -764,10 +759,7 @@ fn update_with_shared_deps() {
         .run();
 
     // Modify a file manually, and commit it
-    File::create(&git_project.root().join("src/bar.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() { println!("hello!"); }"#)
-        .unwrap();
+    git_project.change_file("src/bar.rs", r#"pub fn bar() { println!("hello!"); }"#);
     let repo = git2::Repository::open(&git_project.root()).unwrap();
     let old_head = repo.head().unwrap().target().unwrap();
     git::add(&repo);
@@ -785,11 +777,13 @@ fn update_with_shared_deps() {
         .with_status(101)
         .with_stderr(
             "\
-[UPDATING] git repository [..]
 [ERROR] Unable to update [..]
 
 Caused by:
-  revspec '0.1.2' not found; [..]
+  precise value for git is not a git revision: 0.1.2
+
+Caused by:
+  unable to parse OID - contains invalid characters; class=Invalid (3)
 ",
         )
         .run();
@@ -878,6 +872,7 @@ fn dep_with_submodule() {
         .with_stderr(
             "\
 [UPDATING] git repository [..]
+[UPDATING] git submodule `file://[..]/dep2`
 [COMPILING] dep1 [..]
 [COMPILING] foo [..]
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]\n",
@@ -941,6 +936,7 @@ fn dep_with_bad_submodule() {
     let expected = format!(
         "\
 [UPDATING] git repository [..]
+[UPDATING] git submodule `file://[..]/dep2`
 [ERROR] failed to get `dep1` as a dependency of package `foo v0.5.0 [..]`
 
 Caused by:
@@ -1024,10 +1020,7 @@ fn two_deps_only_update_one() {
         )
         .run();
 
-    File::create(&git1.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn foo() {}"#)
-        .unwrap();
+    git1.change_file("src/lib.rs", "pub fn foo() {}");
     let repo = git2::Repository::open(&git1.root()).unwrap();
     git::add(&repo);
     let oid = git::commit(&repo);
@@ -1084,10 +1077,7 @@ fn stale_cached_version() {
 
     // Update the repo, and simulate someone else updating the lock file and then
     // us pulling it down.
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 1 + 0 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 1 + 0 }");
     let repo = git2::Repository::open(&bar.root()).unwrap();
     git::add(&repo);
     git::commit(&repo);
@@ -1096,29 +1086,26 @@ fn stale_cached_version() {
 
     let rev = repo.revparse_single("HEAD").unwrap().id();
 
-    File::create(&foo.root().join("Cargo.lock"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [[package]]
-        name = "foo"
-        version = "0.0.0"
-        dependencies = [
-         'bar 0.0.0 (git+{url}#{hash})'
-        ]
+    foo.change_file(
+        "Cargo.lock",
+        &format!(
+            r#"
+                [[package]]
+                name = "foo"
+                version = "0.0.0"
+                dependencies = [
+                 'bar 0.0.0 (git+{url}#{hash})'
+                ]
 
-        [[package]]
-        name = "bar"
-        version = "0.0.0"
-        source = 'git+{url}#{hash}'
-    "#,
-                url = bar.url(),
-                hash = rev
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+                [[package]]
+                name = "bar"
+                version = "0.0.0"
+                source = 'git+{url}#{hash}'
+            "#,
+            url = bar.url(),
+            hash = rev
+        ),
+    );
 
     // Now build!
     foo.cargo("build")
@@ -1182,6 +1169,7 @@ fn dep_with_changed_submodule() {
     p.cargo("run")
         .with_stderr(
             "[UPDATING] git repository `[..]`\n\
+             [UPDATING] git submodule `file://[..]/dep2`\n\
              [COMPILING] dep1 v0.5.0 ([..])\n\
              [COMPILING] foo v0.5.0 ([..])\n\
              [FINISHED] dev [unoptimized + debuginfo] target(s) in \
@@ -1191,16 +1179,13 @@ fn dep_with_changed_submodule() {
         .with_stdout("project2\n")
         .run();
 
-    File::create(&git_project.root().join(".gitmodules"))
-        .unwrap()
-        .write_all(
-            format!(
-                "[submodule \"src\"]\n\tpath = src\n\turl={}",
-                git_project3.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    git_project.change_file(
+        ".gitmodules",
+        &format!(
+            "[submodule \"src\"]\n\tpath = src\n\turl={}",
+            git_project3.url()
+        ),
+    );
 
     // Sync the submodule and reset it to the new remote.
     sub.sync().unwrap();
@@ -1229,6 +1214,7 @@ fn dep_with_changed_submodule() {
         .with_stderr("")
         .with_stderr(&format!(
             "[UPDATING] git repository `{}`\n\
+             [UPDATING] git submodule `file://[..]/dep3`\n\
              [UPDATING] dep1 v0.5.0 ([..]) -> #[..]\n\
              ",
             git_project.url()
@@ -1357,7 +1343,7 @@ fn git_build_cmd_freshness() {
 
     // Modify an ignored file and make sure we don't rebuild
     println!("second pass");
-    File::create(&foo.root().join("src/bar.rs")).unwrap();
+    foo.change_file("src/bar.rs", "");
     foo.cargo("build").with_stdout("").run();
 }
 
@@ -1455,10 +1441,7 @@ fn git_repo_changing_no_rebuild() {
         .run();
 
     // Make a commit to lock p2 to a different rev
-    File::create(&bar.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    bar.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     let repo = git2::Repository::open(&bar.root()).unwrap();
     git::add(&repo);
     git::commit(&repo);
@@ -1562,10 +1545,7 @@ fn git_dep_build_cmd() {
     p.process(&p.bin("foo")).with_stdout("0\n").run();
 
     // Touching bar.rs.in should cause the `build` command to run again.
-    fs::File::create(&p.root().join("bar/src/bar.rs.in"))
-        .unwrap()
-        .write_all(b"pub fn gimme() -> i32 { 1 }")
-        .unwrap();
+    p.change_file("bar/src/bar.rs.in", "pub fn gimme() -> i32 { 1 }");
 
     p.cargo("build").run();
 
@@ -1832,23 +1812,20 @@ fn switch_deps_does_not_update_transitive() {
 
     // Update the dependency to point to the second repository, but this
     // shouldn't update the transitive dependency which is the same.
-    File::create(&p.root().join("Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-            [project]
-            name = "foo"
-            version = "0.5.0"
-            authors = []
-            [dependencies.dep]
-            git = '{}'
-    "#,
-                dep2.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    p.change_file(
+        "Cargo.toml",
+        &format!(
+            r#"
+                [project]
+                name = "foo"
+                version = "0.5.0"
+                authors = []
+                [dependencies.dep]
+                git = '{}'
+            "#,
+            dep2.url()
+        ),
+    );
 
     p.cargo("build")
         .with_stderr(&format!(
@@ -1908,19 +1885,12 @@ fn update_one_source_updates_all_packages_in_that_git_source() {
     let rev1 = repo.revparse_single("HEAD").unwrap().id();
 
     // Just be sure to change a file
-    File::create(&dep.root().join("src/lib.rs"))
-        .unwrap()
-        .write_all(br#"pub fn bar() -> i32 { 2 }"#)
-        .unwrap();
+    dep.change_file("src/lib.rs", "pub fn bar() -> i32 { 2 }");
     git::add(&repo);
     git::commit(&repo);
 
     p.cargo("update -p dep").run();
-    let mut lockfile = String::new();
-    File::open(&p.root().join("Cargo.lock"))
-        .unwrap()
-        .read_to_string(&mut lockfile)
-        .unwrap();
+    let lockfile = p.read_lockfile();
     assert!(
         !lockfile.contains(&rev1.to_string()),
         "{} in {}",
@@ -1984,23 +1954,20 @@ fn switch_sources() {
         )
         .run();
 
-    File::create(&p.root().join("b/Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [project]
-        name = "b"
-        version = "0.5.0"
-        authors = []
-        [dependencies.a]
-        git = '{}'
-    "#,
-                a2.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+    p.change_file(
+        "b/Cargo.toml",
+        &format!(
+            r#"
+                [project]
+                name = "b"
+                version = "0.5.0"
+                authors = []
+                [dependencies.a]
+                git = '{}'
+            "#,
+            a2.url()
+        ),
+    );
 
     p.cargo("build")
         .with_stderr(
@@ -2217,24 +2184,21 @@ fn add_a_git_dep() {
 
     p.cargo("build").run();
 
-    File::create(p.root().join("a/Cargo.toml"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
-        [package]
-        name = "a"
-        version = "0.0.1"
-        authors = []
+    p.change_file(
+        "a/Cargo.toml",
+        &format!(
+            r#"
+                [package]
+                name = "a"
+                version = "0.0.1"
+                authors = []
 
-        [dependencies]
-        git = {{ git = '{}' }}
-    "#,
-                git.url()
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+                [dependencies]
+                git = {{ git = '{}' }}
+            "#,
+            git.url()
+        ),
+    );
 
     p.cargo("build").run();
 }
@@ -2580,25 +2544,23 @@ fn templatedir_doesnt_cause_problems() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    File::create(paths::home().join(".gitconfig"))
-        .unwrap()
-        .write_all(
-            format!(
-                r#"
+    fs::write(
+        paths::home().join(".gitconfig"),
+        format!(
+            r#"
                 [init]
                 templatedir = {}
             "#,
-                git_project2
-                    .url()
-                    .to_file_path()
-                    .unwrap()
-                    .to_str()
-                    .unwrap()
-                    .replace("\\", "/")
-            )
-            .as_bytes(),
-        )
-        .unwrap();
+            git_project2
+                .url()
+                .to_file_path()
+                .unwrap()
+                .to_str()
+                .unwrap()
+                .replace("\\", "/")
+        ),
+    )
+    .unwrap();
 
     p.cargo("build").run();
 }
@@ -2823,3 +2785,237 @@ to proceed despite [..]
     git::commit(&repo);
     git_project.cargo("package --no-verify").run();
 }
+
+#[cargo_test]
+fn default_not_master() {
+    let project = project();
+
+    // Create a repository with a `master` branch, but switch the head to a
+    // branch called `main` at the same time.
+    let (git_project, repo) = git::new_repo("dep1", |project| {
+        project
+            .file("Cargo.toml", &basic_lib_manifest("dep1"))
+            .file("src/lib.rs", "pub fn foo() {}")
+    });
+    let head_id = repo.head().unwrap().target().unwrap();
+    let head = repo.find_commit(head_id).unwrap();
+    repo.branch("main", &head, false).unwrap();
+    repo.set_head("refs/heads/main").unwrap();
+
+    // Then create a commit on the new `main` branch so `master` and `main`
+    // differ.
+    git_project.change_file("src/lib.rs", "");
+    git::add(&repo);
+    git::commit(&repo);
+
+    let project = project
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [project]
+                    name = "foo"
+                    version = "0.5.0"
+
+                    [dependencies]
+                    dep1 = {{ git = '{}' }}
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/lib.rs", "pub fn foo() { dep1::foo() }")
+        .build();
+
+    project
+        .cargo("build")
+        .with_stderr(
+            "\
+[UPDATING] git repository `[..]`
+warning: fetching `master` branch from `[..]` but the `HEAD` \
+    reference for this repository is not the \
+    `master` branch. This behavior will change \
+    in Cargo in the future and your build may \
+    break, so it's recommended to place \
+    `branch = \"master\"` in Cargo.toml when \
+    depending on this git repository to ensure \
+    that your build will continue to work.
+[COMPILING] dep1 v0.5.0 ([..])
+[COMPILING] foo v0.5.0 ([..])
+[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn historical_lockfile_works() {
+    let project = project();
+
+    let (git_project, repo) = git::new_repo("dep1", |project| {
+        project
+            .file("Cargo.toml", &basic_lib_manifest("dep1"))
+            .file("src/lib.rs", "")
+    });
+    let head_id = repo.head().unwrap().target().unwrap();
+
+    let project = project
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [project]
+                    name = "foo"
+                    version = "0.5.0"
+
+                    [dependencies]
+                    dep1 = {{ git = '{}', branch = 'master' }}
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    project.cargo("build").run();
+    project.change_file(
+        "Cargo.lock",
+        &format!(
+            r#"# This file is automatically @generated by Cargo.
+# It is not intended for manual editing.
+[[package]]
+name = "dep1"
+version = "0.5.0"
+source = "git+{}#{}"
+
+[[package]]
+name = "foo"
+version = "0.5.0"
+dependencies = [
+ "dep1",
+]
+"#,
+            git_project.url(),
+            head_id
+        ),
+    );
+    project
+        .cargo("build")
+        .with_stderr("[FINISHED] [..]\n")
+        .run();
+}
+
+#[cargo_test]
+fn historical_lockfile_works_with_vendor() {
+    let project = project();
+
+    let (git_project, repo) = git::new_repo("dep1", |project| {
+        project
+            .file("Cargo.toml", &basic_lib_manifest("dep1"))
+            .file("src/lib.rs", "")
+    });
+    let head_id = repo.head().unwrap().target().unwrap();
+
+    let project = project
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [project]
+                    name = "foo"
+                    version = "0.5.0"
+
+                    [dependencies]
+                    dep1 = {{ git = '{}', branch = 'master' }}
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    let output = project.cargo("vendor").exec_with_output().unwrap();
+    project.change_file(".cargo/config", str::from_utf8(&output.stdout).unwrap());
+    project.change_file(
+        "Cargo.lock",
+        &format!(
+            r#"# This file is automatically @generated by Cargo.
+# It is not intended for manual editing.
+[[package]]
+name = "dep1"
+version = "0.5.0"
+source = "git+{}#{}"
+
+[[package]]
+name = "foo"
+version = "0.5.0"
+dependencies = [
+ "dep1",
+]
+"#,
+            git_project.url(),
+            head_id
+        ),
+    );
+    project.cargo("build").run();
+}
+
+#[cargo_test]
+fn two_dep_forms() {
+    let project = project();
+
+    let (git_project, _repo) = git::new_repo("dep1", |project| {
+        project
+            .file("Cargo.toml", &basic_lib_manifest("dep1"))
+            .file("src/lib.rs", "")
+    });
+
+    let project = project
+        .file(
+            "Cargo.toml",
+            &format!(
+                r#"
+                    [project]
+                    name = "foo"
+                    version = "0.5.0"
+
+                    [dependencies]
+                    dep1 = {{ git = '{}', branch = 'master' }}
+                    a = {{ path = 'a' }}
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("src/lib.rs", "")
+        .file(
+            "a/Cargo.toml",
+            &format!(
+                r#"
+                    [project]
+                    name = "a"
+                    version = "0.5.0"
+
+                    [dependencies]
+                    dep1 = {{ git = '{}' }}
+                "#,
+                git_project.url()
+            ),
+        )
+        .file("a/src/lib.rs", "")
+        .build();
+
+    project
+        .cargo("build")
+        .with_stderr(
+            "\
+[UPDATING] [..]
+warning: two git dependencies found for `[..]` where one uses `branch = \"master\"` \
+and the other doesn't; this will break in a future version of Cargo, so please \
+ensure the dependency forms are consistent
+warning: [..]
+[COMPILING] [..]
+[COMPILING] [..]
+[COMPILING] [..]
+[FINISHED] [..]
+",
+        )
+        .run();
+}
