@@ -5,14 +5,15 @@ use std::io::prelude::*;
 
 use cargo_test_support::cross_compile;
 use cargo_test_support::git;
+use cargo_test_support::registry::{registry_path, registry_url, Package};
+use cargo_test_support::{
+    basic_manifest, cargo_process, no_such_file_err_msg, project, symlink_supported, t,
+};
+
 use cargo_test_support::install::{
     assert_has_installed_exe, assert_has_not_installed_exe, cargo_home,
 };
 use cargo_test_support::paths;
-use cargo_test_support::registry::Package;
-use cargo_test_support::{
-    basic_manifest, cargo_process, project, symlink_supported, t, NO_SUCH_FILE_ERR_MSG,
-};
 
 fn pkg(name: &str, vers: &str) {
     Package::new(name, vers)
@@ -51,6 +52,35 @@ fn simple() {
     assert_has_not_installed_exe(cargo_home(), "foo");
 }
 
+#[cargo_test]
+fn with_index() {
+    pkg("foo", "0.0.1");
+
+    cargo_process("install foo --index")
+        .arg(registry_url().to_string())
+        .with_stderr(&format!(
+            "\
+[UPDATING] `{reg}` index
+[DOWNLOADING] crates ...
+[DOWNLOADED] foo v0.0.1 (registry `{reg}`)
+[INSTALLING] foo v0.0.1 (registry `{reg}`)
+[COMPILING] foo v0.0.1 (registry `{reg}`)
+[FINISHED] release [optimized] target(s) in [..]
+[INSTALLING] [CWD]/home/.cargo/bin/foo[EXE]
+[INSTALLED] package `foo v0.0.1 (registry `{reg}`)` (executable `foo[EXE]`)
+[WARNING] be sure to add `[..]` to your PATH to be able to run the installed binaries
+",
+            reg = registry_path().to_str().unwrap()
+        ))
+        .run();
+    assert_has_installed_exe(cargo_home(), "foo");
+
+    cargo_process("uninstall foo")
+        .with_stderr("[REMOVING] [CWD]/home/.cargo/bin/foo[EXE]")
+        .run();
+    assert_has_not_installed_exe(cargo_home(), "foo");
+}
+
 #[cargo_test]
 fn multiple_pkgs() {
     pkg("foo", "0.0.1");
@@ -279,6 +309,35 @@ fn install_path() {
         .run();
 }
 
+#[cargo_test]
+fn install_target_dir() {
+    let p = project().file("src/main.rs", "fn main() {}").build();
+
+    p.cargo("install --target-dir td_test")
+        .with_stderr(
+            "\
+[WARNING] Using `cargo install` [..]
+[INSTALLING] foo v0.0.1 [..]
+[COMPILING] foo v0.0.1 [..]
+[FINISHED] release [..]
+[INSTALLING] [..]foo[EXE]
+[INSTALLED] package `foo v0.0.1 [..]foo[..]` (executable `foo[EXE]`)
+[WARNING] be sure to add [..]
+",
+        )
+        .run();
+
+    let mut path = p.root();
+    path.push("td_test");
+    assert!(path.exists());
+
+    #[cfg(not(windows))]
+    path.push("release/foo");
+    #[cfg(windows)]
+    path.push("release/foo.exe");
+    assert!(path.exists());
+}
+
 #[cargo_test]
 fn multiple_crates_error() {
     let p = git::repo(&paths::root().join("foo"))
@@ -845,7 +904,7 @@ fn uninstall_cwd_no_project() {
 
 Caused by:
   {err_msg}",
-            err_msg = NO_SUCH_FILE_ERR_MSG,
+            err_msg = no_such_file_err_msg(),
         ))
         .run();
 }
@@ -1497,3 +1556,15 @@ fn install_git_with_symlink_home() {
         )
         .run();
 }
+
+#[cargo_test]
+fn install_yanked_cargo_package() {
+    Package::new("baz", "0.0.1").yanked(true).publish();
+    cargo_process("install baz --version 0.0.1")
+        .with_status(101)
+        .with_stderr_contains(
+            "error: cannot install package `baz`, it has been yanked from registry \
+         `https://github.com/rust-lang/crates.io-index`",
+        )
+        .run();
+}
