@@ -1,12 +1,10 @@
 //! Tests for the `cargo publish` command.
 
-use std::fs::{self, File};
-use std::io::prelude::*;
-
 use cargo_test_support::git::{self, repo};
 use cargo_test_support::paths;
 use cargo_test_support::registry::{self, registry_path, registry_url, Package};
-use cargo_test_support::{basic_manifest, project, publish};
+use cargo_test_support::{basic_manifest, no_such_file_err_msg, project, publish};
+use std::fs;
 
 const CLEAN_FOO_JSON: &str = r#"
     {
@@ -91,8 +89,7 @@ fn simple() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-verify --token sekrit")
         .with_stderr(&format!(
             "\
 [UPDATING] `{reg}` index
@@ -133,22 +130,23 @@ fn old_token_location() {
     fs::remove_file(&credentials).unwrap();
 
     // Verify can't publish without a token.
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-verify")
         .with_status(101)
-        .with_stderr_contains("[ERROR] no upload token found, please run `cargo login`")
+        .with_stderr_contains(
+            "[ERROR] no upload token found, \
+            please run `cargo login` or pass `--token`",
+        )
         .run();
 
-    File::create(&credentials)
-        .unwrap()
-        .write_all(br#"token = "api-token""#)
-        .unwrap();
+    fs::write(&credentials, r#"token = "api-token""#).unwrap();
 
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-verify")
         .with_stderr(&format!(
             "\
 [UPDATING] `{reg}` index
+[WARNING] using `registry.token` config value with source replacement is deprecated
+This may become a hard error in the future[..]
+Use the --token command-line flag to remove this warning.
 [WARNING] manifest has no documentation, [..]
 See [..]
 [PACKAGING] foo v0.0.1 ([CWD])
@@ -182,7 +180,7 @@ fn simple_with_host() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --no-verify --host")
+    p.cargo("publish --no-verify --token sekrit --host")
         .arg(registry_url().to_string())
         .with_stderr(&format!(
             "\
@@ -229,7 +227,7 @@ fn simple_with_index_and_host() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --no-verify --index")
+    p.cargo("publish --no-verify --token sekrit --index")
         .arg(registry_url().to_string())
         .arg("--host")
         .arg(registry_url().to_string())
@@ -279,8 +277,7 @@ fn git_deps() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish -v --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish -v --no-verify --token sekrit")
         .with_status(101)
         .with_stderr(
             "\
@@ -318,8 +315,7 @@ fn path_dependency_no_version() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --token sekrit")
         .with_status(101)
         .with_stderr(
             "\
@@ -388,8 +384,7 @@ fn dont_publish_dirty() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --token sekrit")
         .with_status(101)
         .with_stderr(
             "\
@@ -429,9 +424,7 @@ fn publish_clean() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --token sekrit").run();
 
     validate_upload_foo_clean();
 }
@@ -460,11 +453,7 @@ fn publish_in_sub_repo() {
         .file("bar/src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish")
-        .cwd("bar")
-        .arg("--index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --token sekrit").cwd("bar").run();
 
     validate_upload_foo_clean();
 }
@@ -494,9 +483,7 @@ fn publish_when_ignored() {
         .file(".gitignore", "baz")
         .build();
 
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --token sekrit").run();
 
     publish::validate_upload(
         CLEAN_FOO_JSON,
@@ -535,11 +522,7 @@ fn ignore_when_crate_ignored() {
         "#,
         )
         .nocommit_file("bar/src/main.rs", "fn main() {}");
-    p.cargo("publish")
-        .cwd("bar")
-        .arg("--index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --token sekrit").cwd("bar").run();
 
     publish::validate_upload(
         CLEAN_FOO_JSON,
@@ -576,8 +559,7 @@ fn new_crate_rejected() {
         "#,
         )
         .nocommit_file("src/main.rs", "fn main() {}");
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --token sekrit")
         .with_status(101)
         .with_stderr_contains(
             "[ERROR] 3 files in the working directory contain \
@@ -733,6 +715,82 @@ fn publish_allowed_registry() {
     );
 }
 
+#[cargo_test]
+fn publish_implicitly_to_only_allowed_registry() {
+    registry::init();
+
+    let p = project().build();
+
+    let _ = repo(&paths::root().join("foo"))
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            documentation = "foo"
+            homepage = "foo"
+            repository = "foo"
+            publish = ["alternative"]
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("publish").run();
+
+    publish::validate_alt_upload(
+        CLEAN_FOO_JSON,
+        "foo-0.0.1.crate",
+        &[
+            "Cargo.lock",
+            "Cargo.toml",
+            "Cargo.toml.orig",
+            "src/main.rs",
+            ".cargo_vcs_info.json",
+        ],
+    );
+}
+
+#[cargo_test]
+fn publish_fail_with_no_registry_specified() {
+    registry::init();
+
+    let p = project().build();
+
+    let _ = repo(&paths::root().join("foo"))
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            documentation = "foo"
+            homepage = "foo"
+            repository = "foo"
+            publish = ["alternative", "test"]
+        "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .build();
+
+    p.cargo("publish")
+        .with_status(101)
+        .with_stderr(
+            "\
+[ERROR] `foo` cannot be published.
+The registry `crates-io` is not listed in the `publish` value in Cargo.toml.
+",
+        )
+        .run();
+}
+
 #[cargo_test]
 fn block_publish_no_registry() {
     registry::init();
@@ -826,8 +884,7 @@ fn publish_with_select_features() {
         )
         .build();
 
-    p.cargo("publish --features required --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --features required --token sekrit")
         .with_stderr_contains("[UPLOADING] foo v0.0.1 ([CWD])")
         .run();
 }
@@ -860,8 +917,7 @@ fn publish_with_all_features() {
         )
         .build();
 
-    p.cargo("publish --all-features --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --all-features --token sekrit")
         .with_stderr_contains("[UPLOADING] foo v0.0.1 ([CWD])")
         .run();
 }
@@ -894,8 +950,7 @@ fn publish_with_no_default_features() {
         )
         .build();
 
-    p.cargo("publish --no-default-features --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-default-features --token sekrit")
         .with_stderr_contains("error: This crate requires `required` feature!")
         .with_status(101)
         .run();
@@ -936,8 +991,7 @@ fn publish_with_patch() {
     p.cargo("build").run();
 
     // Check that verify fails with patched crate which has new functionality.
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --token sekrit")
         .with_stderr_contains("[..]newfunc[..]")
         .with_status(101)
         .run();
@@ -945,9 +999,7 @@ fn publish_with_patch() {
     // Remove the usage of new functionality and try again.
     p.change_file("src/main.rs", "extern crate bar; pub fn main() {}");
 
-    p.cargo("publish --index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --token sekrit").run();
 
     // Note, use of `registry` in the deps here is an artifact that this
     // publishes to a fake, local registry that is pretending to be crates.io.
@@ -1015,7 +1067,10 @@ fn publish_checks_for_token_before_verify() {
     // Assert upload token error before the package is verified
     p.cargo("publish")
         .with_status(101)
-        .with_stderr_contains("[ERROR] no upload token found, please run `cargo login`")
+        .with_stderr_contains(
+            "[ERROR] no upload token found, \
+            please run `cargo login` or pass `--token`",
+        )
         .with_stderr_does_not_contain("[VERIFYING] foo v0.0.1 ([CWD])")
         .run();
 
@@ -1042,7 +1097,7 @@ fn publish_with_bad_source() {
         .file("src/lib.rs", "")
         .build();
 
-    p.cargo("publish")
+    p.cargo("publish --token sekrit")
         .with_status(101)
         .with_stderr(
             "\
@@ -1063,7 +1118,7 @@ Check for a source-replacement in .cargo/config.
         "#,
     );
 
-    p.cargo("publish")
+    p.cargo("publish --token sekrit")
         .with_status(101)
         .with_stderr(
             "\
@@ -1117,9 +1172,7 @@ fn publish_git_with_version() {
         .build();
 
     p.cargo("run").with_stdout("2").run();
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
-        .run();
+    p.cargo("publish --no-verify --token sekrit").run();
 
     publish::validate_upload_with_contents(
         r#"
@@ -1208,8 +1261,7 @@ fn publish_dev_dep_no_version() {
         .file("bar/src/lib.rs", "")
         .build();
 
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-verify --token sekrit")
         .with_stderr(
             "\
 [UPDATING] [..]
@@ -1284,8 +1336,7 @@ fn credentials_ambiguous_filename() {
         .file("src/main.rs", "fn main() {}")
         .build();
 
-    p.cargo("publish --no-verify --index")
-        .arg(registry_url().to_string())
+    p.cargo("publish --no-verify --token sekrit")
         .with_stderr_contains(
             "\
 [WARNING] Both `[..]/credentials` and `[..]/credentials.toml` exist. Using `[..]/credentials`
@@ -1295,3 +1346,114 @@ fn credentials_ambiguous_filename() {
 
     validate_upload_foo();
 }
+
+#[cargo_test]
+fn index_requires_token() {
+    // --index will not load registry.token to avoid possibly leaking
+    // crates.io token to another server.
+    registry::init();
+    let credentials = paths::home().join(".cargo/credentials");
+    fs::remove_file(&credentials).unwrap();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("publish --no-verify --index")
+        .arg(registry_url().to_string())
+        .with_status(101)
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[ERROR] command-line argument --index requires --token to be specified
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn registry_token_with_source_replacement() {
+    // publish with source replacement without --token
+    registry::init();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+        "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("publish --no-verify")
+        .with_stderr(
+            "\
+[UPDATING] [..]
+[WARNING] using `registry.token` config value with source replacement is deprecated
+This may become a hard error in the future[..]
+Use the --token command-line flag to remove this warning.
+[WARNING] manifest has no documentation, [..]
+See [..]
+[PACKAGING] foo v0.0.1 ([CWD])
+[UPLOADING] foo v0.0.1 ([CWD])
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn publish_with_missing_readme() {
+    registry::init();
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+                [package]
+                name = "foo"
+                version = "0.1.0"
+                authors = []
+                license = "MIT"
+                description = "foo"
+                homepage = "https://example.com/"
+                readme = "foo.md"
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("publish --no-verify --token sekrit")
+        .with_status(101)
+        .with_stderr(&format!(
+            "\
+[UPDATING] [..]
+[PACKAGING] foo v0.1.0 [..]
+[UPLOADING] foo v0.1.0 [..]
+[ERROR] failed to read `readme` file for package `foo v0.1.0 ([ROOT]/foo)`
+
+Caused by:
+  failed to read `[ROOT]/foo/foo.md`
+
+Caused by:
+  {}
+",
+            no_such_file_err_msg()
+        ))
+        .run();
+}
