@@ -129,7 +129,7 @@ fn cargo_test_overflow_checks() {
             use std::panic;
             pub fn main() {
                 let r = panic::catch_unwind(|| {
-                    [1, i32::max_value()].iter().sum::<i32>();
+                    [1, i32::MAX].iter().sum::<i32>();
                 });
                 assert!(r.is_err());
             }"#,
@@ -1984,7 +1984,7 @@ fn bin_is_preserved() {
     p.cargo("build -v").run();
     assert!(p.bin("foo").is_file());
 
-    println!("testing");
+    println!("test");
     p.cargo("test -v").run();
     assert!(p.bin("foo").is_file());
 }
@@ -3358,11 +3358,14 @@ fn json_artifact_includes_test_flag() {
             "doctest": true,
             "edition": "2015",
             "name":"foo",
-            "src_path":"[..]lib.rs"
+            "src_path":"[..]lib.rs",
+            "test": true
         },
-        "filenames":["[..]/foo-[..]"],
+        "filenames":"{...}",
         "fresh": false
     }
+
+    {"reason": "build-finished", "success": true}
 "#,
         )
         .run();
@@ -3379,7 +3382,7 @@ fn json_artifact_includes_executable_for_library_tests() {
         .with_json(
             r#"
             {
-                "executable": "[..]/foo/target/debug/foo-[..][EXE]",
+                "executable": "[..]/foo/target/debug/deps/foo-[..][EXE]",
                 "features": [],
                 "filenames": "{...}",
                 "fresh": false,
@@ -3392,9 +3395,12 @@ fn json_artifact_includes_executable_for_library_tests() {
                     "doctest": true,
                     "edition": "2015",
                     "name": "foo",
-                    "src_path": "[..]/foo/src/lib.rs"
+                    "src_path": "[..]/foo/src/lib.rs",
+                    "test": true
                 }
             }
+
+            {"reason": "build-finished", "success": true}
         "#,
         )
         .run();
@@ -3413,7 +3419,7 @@ fn json_artifact_includes_executable_for_integration_tests() {
         .with_json(
             r#"
             {
-                "executable": "[..]/foo/target/debug/integration_test-[..][EXE]",
+                "executable": "[..]/foo/target/debug/deps/integration_test-[..][EXE]",
                 "features": [],
                 "filenames": "{...}",
                 "fresh": false,
@@ -3426,9 +3432,12 @@ fn json_artifact_includes_executable_for_integration_tests() {
                     "doctest": false,
                     "edition": "2015",
                     "name": "integration_test",
-                    "src_path": "[..]/foo/tests/integration_test.rs"
+                    "src_path": "[..]/foo/tests/integration_test.rs",
+                    "test": true
                 }
             }
+
+            {"reason": "build-finished", "success": true}
         "#,
         )
         .run();
