@@ -1,7 +1,8 @@
-use crate::support::rustc_host;
-use crate::support::{basic_lib_manifest, project};
+//! Tests for configuration values that point to programs.
 
-#[test]
+use cargo_test_support::{basic_lib_manifest, no_such_file_err_msg, project, rustc_host};
+
+#[cargo_test]
 fn pathless_tools() {
     let target = rustc_host();
 
@@ -13,7 +14,6 @@ fn pathless_tools() {
             &format!(
                 r#"
             [target.{}]
-            ar = "nonexistent-ar"
             linker = "nonexistent-linker"
         "#,
                 target
@@ -25,25 +25,22 @@ fn pathless_tools() {
         .with_stderr(
             "\
 [COMPILING] foo v0.5.0 ([CWD])
-[RUNNING] `rustc [..] -C ar=nonexistent-ar -C linker=nonexistent-linker [..]`
+[RUNNING] `rustc [..] -C linker=nonexistent-linker [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
         )
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn absolute_tools() {
     let target = rustc_host();
 
     // Escaped as they appear within a TOML config file
-    let config = if cfg!(windows) {
-        (
-            r#"C:\\bogus\\nonexistent-ar"#,
-            r#"C:\\bogus\\nonexistent-linker"#,
-        )
+    let linker = if cfg!(windows) {
+        r#"C:\\bogus\\nonexistent-linker"#
     } else {
-        (r#"/bogus/nonexistent-ar"#, r#"/bogus/nonexistent-linker"#)
+        r#"/bogus/nonexistent-linker"#
     };
 
     let foo = project()
@@ -54,32 +51,34 @@ fn absolute_tools() {
             &format!(
                 r#"
             [target.{target}]
-            ar = "{ar}"
             linker = "{linker}"
         "#,
                 target = target,
-                ar = config.0,
-                linker = config.1
+                linker = linker
             ),
         )
         .build();
 
-    foo.cargo("build --verbose").with_stderr("\
+    foo.cargo("build --verbose")
+        .with_stderr(
+            "\
 [COMPILING] foo v0.5.0 ([CWD])
-[RUNNING] `rustc [..] -C ar=[ROOT]bogus/nonexistent-ar -C linker=[ROOT]bogus/nonexistent-linker [..]`
+[RUNNING] `rustc [..] -C linker=[..]bogus/nonexistent-linker [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
-").run();
+",
+        )
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn relative_tools() {
     let target = rustc_host();
 
     // Escaped as they appear within a TOML config file
-    let config = if cfg!(windows) {
-        (r#".\\nonexistent-ar"#, r#".\\tools\\nonexistent-linker"#)
+    let linker = if cfg!(windows) {
+        r#".\\tools\\nonexistent-linker"#
     } else {
-        (r#"./nonexistent-ar"#, r#"./tools/nonexistent-linker"#)
+        r#"./tools/nonexistent-linker"#
     };
 
     // Funky directory structure to test that relative tool paths are made absolute
@@ -93,29 +92,30 @@ fn relative_tools() {
             &format!(
                 r#"
             [target.{target}]
-            ar = "{ar}"
             linker = "{linker}"
         "#,
                 target = target,
-                ar = config.0,
-                linker = config.1
+                linker = linker
             ),
         )
         .build();
 
     let prefix = p.root().into_os_string().into_string().unwrap();
 
-    p.cargo("build --verbose").cwd("bar").with_stderr(&format!(
+    p.cargo("build --verbose")
+        .cwd("bar")
+        .with_stderr(&format!(
             "\
 [COMPILING] bar v0.5.0 ([CWD])
-[RUNNING] `rustc [..] -C ar={prefix}/./nonexistent-ar -C linker={prefix}/./tools/nonexistent-linker [..]`
+[RUNNING] `rustc [..] -C linker={prefix}/./tools/nonexistent-linker [..]`
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 ",
             prefix = prefix,
-        )).run();
+        ))
+        .run();
 }
 
-#[test]
+#[cargo_test]
 fn custom_runner() {
     let target = rustc_host();
 
@@ -152,7 +152,7 @@ fn custom_runner() {
             "\
 [COMPILING] foo v0.0.1 ([CWD])
 [RUNNING] `rustc [..]`
-[FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
+[FINISHED] test [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `nonexistent-runner -r [..]/target/debug/deps/test-[..][EXE] --param`
 ",
         )
@@ -165,7 +165,7 @@ fn custom_runner() {
 [COMPILING] foo v0.0.1 ([CWD])
 [RUNNING] `rustc [..]`
 [RUNNING] `rustc [..]`
-[FINISHED] release [optimized] target(s) in [..]
+[FINISHED] bench [optimized] target(s) in [..]
 [RUNNING] `nonexistent-runner -r [..]/target/release/deps/bench-[..][EXE] --param --bench`
 ",
         )
@@ -173,7 +173,7 @@ fn custom_runner() {
 }
 
 // can set a custom runner via `target.'cfg(..)'.runner`
-#[test]
+#[cargo_test]
 fn custom_runner_cfg() {
     let p = project()
         .file("src/main.rs", "fn main() {}")
@@ -198,8 +198,8 @@ fn custom_runner_cfg() {
         .run();
 }
 
-// custom runner set via `target.$triple.runner` have precende over `target.'cfg(..)'.runner`
-#[test]
+// custom runner set via `target.$triple.runner` have precedence over `target.'cfg(..)'.runner`
+#[cargo_test]
 fn custom_runner_cfg_precedence() {
     let target = rustc_host();
 
@@ -224,7 +224,7 @@ fn custom_runner_cfg_precedence() {
         .with_status(101)
         .with_stderr_contains(
             "\
-            [COMPILING] foo v0.0.1 ([CWD])
+[COMPILING] foo v0.0.1 ([CWD])
 [FINISHED] dev [unoptimized + debuginfo] target(s) in [..]
 [RUNNING] `nonexistent-runner -r target/debug/foo[EXE] --param`
 ",
@@ -232,7 +232,7 @@ fn custom_runner_cfg_precedence() {
         .run();
 }
 
-#[test]
+#[cargo_test]
 fn custom_runner_cfg_collision() {
     let p = project()
         .file("src/main.rs", "fn main() {}")
@@ -250,9 +250,143 @@ fn custom_runner_cfg_collision() {
 
     p.cargo("run -- --param")
         .with_status(101)
-        .with_stderr_contains(
+        .with_stderr(
             "\
 [ERROR] several matching instances of `target.'cfg(..)'.runner` in `.cargo/config`
+first match `cfg(not(target_arch = \"avr\"))` located in [..]/foo/.cargo/config
+second match `cfg(not(target_os = \"none\"))` located in [..]/foo/.cargo/config
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn custom_runner_env() {
+    let target = rustc_host();
+    let p = project().file("src/main.rs", "fn main() {}").build();
+
+    let key = format!(
+        "CARGO_TARGET_{}_RUNNER",
+        target.to_uppercase().replace('-', "_")
+    );
+
+    p.cargo("run")
+        .env(&key, "nonexistent-runner --foo")
+        .with_status(101)
+        .with_stderr(&format!(
+            "\
+[COMPILING] foo [..]
+[FINISHED] dev [..]
+[RUNNING] `nonexistent-runner --foo target/debug/foo[EXE]`
+[ERROR] could not execute process `nonexistent-runner --foo target/debug/foo[EXE]` (never executed)
+
+Caused by:
+  {}
+",
+            no_such_file_err_msg()
+        ))
+        .run();
+}
+
+#[cargo_test]
+fn custom_runner_env_overrides_config() {
+    let target = rustc_host();
+    let p = project()
+        .file("src/main.rs", "fn main() {}")
+        .file(
+            ".cargo/config.toml",
+            &format!(
+                r#"
+        [target.{}]
+        runner = "should-not-run -r"
+    "#,
+                target
+            ),
+        )
+        .build();
+
+    let key = format!(
+        "CARGO_TARGET_{}_RUNNER",
+        target.to_uppercase().replace('-', "_")
+    );
+
+    p.cargo("run")
+        .env(&key, "should-run --foo")
+        .with_status(101)
+        .with_stderr_contains("[RUNNING] `should-run --foo target/debug/foo[EXE]`")
+        .run();
+}
+
+#[cargo_test]
+#[cfg(unix)] // Assumes `true` is in PATH.
+fn custom_runner_env_true() {
+    // Check for a bug where "true" was interpreted as a boolean instead of
+    // the executable.
+    let target = rustc_host();
+    let p = project().file("src/main.rs", "fn main() {}").build();
+
+    let key = format!(
+        "CARGO_TARGET_{}_RUNNER",
+        target.to_uppercase().replace('-', "_")
+    );
+
+    p.cargo("run")
+        .env(&key, "true")
+        .with_stderr_contains("[RUNNING] `true target/debug/foo[EXE]`")
+        .run();
+}
+
+#[cargo_test]
+fn custom_linker_env() {
+    let target = rustc_host();
+    let p = project().file("src/main.rs", "fn main() {}").build();
+
+    let key = format!(
+        "CARGO_TARGET_{}_LINKER",
+        target.to_uppercase().replace('-', "_")
+    );
+
+    p.cargo("build -v")
+        .env(&key, "nonexistent-linker")
+        .with_status(101)
+        .with_stderr_contains("[RUNNING] `rustc [..]-C linker=nonexistent-linker [..]")
+        .run();
+}
+
+#[cargo_test]
+fn cfg_ignored_fields() {
+    // Test for some ignored fields in [target.'cfg()'] tables.
+    let p = project()
+        .file(
+            ".cargo/config",
+            r#"
+            # Try some empty tables.
+            [target.'cfg(not(foo))']
+            [target.'cfg(not(bar))'.somelib]
+
+            # A bunch of unused fields.
+            [target.'cfg(not(target_os = "none"))']
+            linker = 'false'
+            ar = 'false'
+            foo = {rustc-flags = "-l foo"}
+            invalid = 1
+            runner = 'false'
+            rustflags = ''
+            "#,
+        )
+        .file("src/lib.rs", "")
+        .build();
+
+    p.cargo("check")
+        .with_stderr(
+            "\
+[WARNING] unused key `somelib` in [target] config table `cfg(not(bar))`
+[WARNING] unused key `ar` in [target] config table `cfg(not(target_os = \"none\"))`
+[WARNING] unused key `foo` in [target] config table `cfg(not(target_os = \"none\"))`
+[WARNING] unused key `invalid` in [target] config table `cfg(not(target_os = \"none\"))`
+[WARNING] unused key `linker` in [target] config table `cfg(not(target_os = \"none\"))`
+[CHECKING] foo v0.0.1 ([..])
+[FINISHED] [..]
 ",
         )
         .run();
