@@ -5,18 +5,24 @@ use std::borrow::Cow;
 use std::cell::RefCell;
 use std::num::NonZeroU64;
 use std::rc::Rc;
+use std::time::Instant;
+use std::fmt;
 
+use log::trace;
 use rand::rngs::StdRng;
 
 use rustc_data_structures::fx::FxHashMap;
-use rustc::mir;
-use rustc::ty::{
-    self,
-    layout::{LayoutOf, Size},
-    Ty,
+use rustc_middle::{
+    mir,
+    ty::{
+        self,
+        layout::{LayoutCx, LayoutError, TyAndLayout},
+        TyCtxt,
+    },
 };
-use rustc_ast::attr;
-use rustc_span::{source_map::Span, symbol::{sym, Symbol}};
+use rustc_span::symbol::{sym, Symbol};
+use rustc_span::def_id::DefId;
+use rustc_target::abi::{LayoutOf, Size};
 
 use crate::*;
 
@@ -32,11 +38,10 @@ pub struct FrameData<'tcx> {
     /// Extra data for Stacked Borrows.
     pub call_id: stacked_borrows::CallId,
 
-    /// If this is Some(), then this is a special "catch unwind" frame (the frame of the closure
-    /// called by `__rustc_maybe_catch_panic`). When this frame is popped during unwinding a panic,
-    /// we stop unwinding, use the `CatchUnwindData` to
-    /// store the panic payload, and continue execution in the parent frame.
-    pub catch_panic: Option<CatchUnwindData<'tcx>>,
+    /// If this is Some(), then this is a special "catch unwind" frame (the frame of `try_fn`
+    /// called by `try`). When this frame is popped during unwinding a panic,
+    /// we stop unwinding, use the `CatchUnwindData` to handle catching.
+    pub catch_unwind: Option<CatchUnwindData<'tcx>>,
 }
 
 /// Extra memory kinds
@@ -48,10 +53,20 @@ pub enum MiriMemoryKind {
     C,
     /// Windows `HeapAlloc` memory.
     WinHeap,
-    /// Memory for env vars and args, errno, extern statics and other parts of the machine-managed environment.
+    /// Memory for args, errno, and other parts of the machine-managed environment.
+    /// This memory may leak.
     Machine,
-    /// Rust statics.
-    Static,
+    /// Memory for env vars. Separate from `Machine` because we clean it up and leak-check it.
+    Env,
+    /// Globals copied from `tcx`.
+    /// This memory may leak.
+    Global,
+    /// Memory for extern statics.
+    /// This memory may leak.
+    ExternStatic,
+    /// Memory for thread-local statics.
+    /// This memory may leak.
+    Tls,
 }
 
 impl Into<MemoryKind<MiriMemoryKind>> for MiriMemoryKind {
@@ -61,6 +76,33 @@ impl Into<MemoryKind<MiriMemoryKind>> for MiriMemoryKind {
     }
 }
 
+impl MayLeak for MiriMemoryKind {
+    #[inline(always)]
+    fn may_leak(self) -> bool {
+        use self::MiriMemoryKind::*;
+        match self {
+            Rust | C | WinHeap | Env => false,
+            Machine | Global | ExternStatic | Tls => true,
+        }
+    }
+}
+
+impl fmt::Display for MiriMemoryKind {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        use self::MiriMemoryKind::*;
+        match self {
+            Rust => write!(f, "Rust heap"),
+            C => write!(f, "C heap"),
+            WinHeap => write!(f, "Windows heap"),
+            Machine => write!(f, "machine-managed memory"),
+            Env => write!(f, "environment variable"),
+            Global => write!(f, "global (static or const)"),
+            ExternStatic => write!(f, "extern static"),
+            Tls =>  write!(f, "thread-local static"),
+        }
+    }
+}
+
 /// Extra per-allocation data
 #[derive(Debug, Clone)]
 pub struct AllocExtra {
@@ -75,17 +117,31 @@ pub struct MemoryExtra {
     pub intptrcast: intptrcast::MemoryExtra,
 
     /// Mapping extern static names to their canonical allocation.
-    pub(crate) extern_statics: FxHashMap<Symbol, AllocId>,
+    extern_statics: FxHashMap<Symbol, AllocId>,
 
     /// The random number generator used for resolving non-determinism.
     /// Needs to be queried by ptr_to_int, hence needs interior mutability.
     pub(crate) rng: RefCell<StdRng>,
+
+    /// An allocation ID to report when it is being allocated
+    /// (helps for debugging memory leaks and use after free bugs).
+    tracked_alloc_id: Option<AllocId>,
+
+    /// Controls whether alignment of memory accesses is being checked.
+    pub(crate) check_alignment: AlignmentCheck,
 }
 
 impl MemoryExtra {
-    pub fn new(rng: StdRng, stacked_borrows: bool, tracked_pointer_tag: Option<PtrId>) -> Self {
+    pub fn new(
+        rng: StdRng,
+        stacked_borrows: bool,
+        tracked_pointer_tag: Option<PtrId>,
+        tracked_call_id: Option<CallId>,
+        tracked_alloc_id: Option<AllocId>,
+        check_alignment: AlignmentCheck,
+    ) -> Self {
         let stacked_borrows = if stacked_borrows {
-            Some(Rc::new(RefCell::new(stacked_borrows::GlobalState::new(tracked_pointer_tag))))
+            Some(Rc::new(RefCell::new(stacked_borrows::GlobalState::new(tracked_pointer_tag, tracked_call_id))))
         } else {
             None
         };
@@ -94,37 +150,84 @@ impl MemoryExtra {
             intptrcast: Default::default(),
             extern_statics: FxHashMap::default(),
             rng: RefCell::new(rng),
+            tracked_alloc_id,
+            check_alignment,
         }
     }
 
+    fn add_extern_static<'tcx, 'mir>(
+        this: &mut MiriEvalContext<'mir, 'tcx>,
+        name: &str,
+        ptr: Scalar<Tag>,
+    ) {
+        let ptr = ptr.assert_ptr();
+        assert_eq!(ptr.offset, Size::ZERO);
+        this.memory
+            .extra
+            .extern_statics
+            .insert(Symbol::intern(name), ptr.alloc_id)
+            .unwrap_none();
+    }
+
     /// Sets up the "extern statics" for this machine.
-    pub fn init_extern_statics<'mir, 'tcx>(
+    pub fn init_extern_statics<'tcx, 'mir>(
         this: &mut MiriEvalContext<'mir, 'tcx>,
     ) -> InterpResult<'tcx> {
         match this.tcx.sess.target.target.target_os.as_str() {
             "linux" => {
                 // "__cxa_thread_atexit_impl"
                 // This should be all-zero, pointer-sized.
-                let layout = this.layout_of(this.tcx.types.usize)?;
-                let place = this.allocate(layout, MiriMemoryKind::Machine.into());
-                this.write_scalar(Scalar::from_machine_usize(0, &*this.tcx), place.into())?;
-                this.memory
-                    .extra
-                    .extern_statics
-                    .insert(Symbol::intern("__cxa_thread_atexit_impl"), place.ptr.assert_ptr().alloc_id)
-                    .unwrap_none();
+                let layout = this.machine.layouts.usize;
+                let place = this.allocate(layout, MiriMemoryKind::ExternStatic.into());
+                this.write_scalar(Scalar::from_machine_usize(0, this), place.into())?;
+                Self::add_extern_static(this, "__cxa_thread_atexit_impl", place.ptr);
+                // "environ"
+                Self::add_extern_static(this, "environ", this.machine.env_vars.environ.unwrap().ptr);
             }
-            _ => {} // No "extern statics" supported on this platform
+            "windows" => {
+                // "_tls_used"
+                // This is some obscure hack that is part of the Windows TLS story. It's a `u8`.
+                let layout = this.machine.layouts.u8;
+                let place = this.allocate(layout, MiriMemoryKind::ExternStatic.into());
+                this.write_scalar(Scalar::from_u8(0), place.into())?;
+                Self::add_extern_static(this, "_tls_used", place.ptr);
+            }
+            _ => {} // No "extern statics" supported on this target
         }
         Ok(())
     }
 }
 
+/// Precomputed layouts of primitive types
+pub struct PrimitiveLayouts<'tcx> {
+    pub unit: TyAndLayout<'tcx>,
+    pub i8: TyAndLayout<'tcx>,
+    pub i32: TyAndLayout<'tcx>,
+    pub isize: TyAndLayout<'tcx>,
+    pub u8: TyAndLayout<'tcx>,
+    pub u32: TyAndLayout<'tcx>,
+    pub usize: TyAndLayout<'tcx>,
+}
+
+impl<'mir, 'tcx: 'mir> PrimitiveLayouts<'tcx> {
+    fn new(layout_cx: LayoutCx<'tcx, TyCtxt<'tcx>>) -> Result<Self, LayoutError<'tcx>> {
+        Ok(Self {
+            unit: layout_cx.layout_of(layout_cx.tcx.mk_unit())?,
+            i8: layout_cx.layout_of(layout_cx.tcx.types.i8)?,
+            i32: layout_cx.layout_of(layout_cx.tcx.types.i32)?,
+            isize: layout_cx.layout_of(layout_cx.tcx.types.isize)?,
+            u8: layout_cx.layout_of(layout_cx.tcx.types.u8)?,
+            u32: layout_cx.layout_of(layout_cx.tcx.types.u32)?,
+            usize: layout_cx.layout_of(layout_cx.tcx.types.usize)?,
+        })
+    }
+}
+
 /// The machine itself.
-pub struct Evaluator<'tcx> {
+pub struct Evaluator<'mir, 'tcx> {
     /// Environment variables set by `setenv`.
     /// Miri does not expose env vars from the host to the emulated program.
-    pub(crate) env_vars: EnvVars,
+    pub(crate) env_vars: EnvVars<'tcx>,
 
     /// Program arguments (`Option` because we can only initialize them after creating the ecx).
     /// These are *pointers* to argc/argv because macOS.
@@ -146,16 +249,35 @@ pub struct Evaluator<'tcx> {
     /// Whether to enforce the validity invariant.
     pub(crate) validate: bool,
 
-    pub(crate) file_handler: FileHandler,
-    pub(crate) dir_handler: DirHandler,
+    pub(crate) file_handler: shims::posix::FileHandler,
+    pub(crate) dir_handler: shims::posix::DirHandler,
 
     /// The temporary used for storing the argument of
     /// the call to `miri_start_panic` (the panic payload) when unwinding.
-    pub(crate) panic_payload: Option<ImmTy<'tcx, Tag>>,
+    /// This is pointer-sized, and matches the `Payload` type in `src/libpanic_unwind/miri.rs`.
+    pub(crate) panic_payload: Option<Scalar<Tag>>,
+
+    /// The "time anchor" for this machine's monotone clock (for `Instant` simulation).
+    pub(crate) time_anchor: Instant,
+
+    /// The set of threads.
+    pub(crate) threads: ThreadManager<'mir, 'tcx>,
+
+    /// Precomputed `TyLayout`s for primitive data types that are commonly used inside Miri.
+    pub(crate) layouts: PrimitiveLayouts<'tcx>,
+
+    /// Allocations that are considered roots of static memory (that may leak).
+    pub(crate) static_roots: Vec<AllocId>,
 }
 
-impl<'tcx> Evaluator<'tcx> {
-    pub(crate) fn new(communicate: bool, validate: bool) -> Self {
+impl<'mir, 'tcx> Evaluator<'mir, 'tcx> {
+    pub(crate) fn new(
+        communicate: bool,
+        validate: bool,
+        layout_cx: LayoutCx<'tcx, TyCtxt<'tcx>>,
+    ) -> Self {
+        let layouts = PrimitiveLayouts::new(layout_cx)
+            .expect("Couldn't get layouts of primitive types");
         Evaluator {
             // `env_vars` could be initialized properly here if `Memory` were available before
             // calling this method.
@@ -170,12 +292,16 @@ impl<'tcx> Evaluator<'tcx> {
             file_handler: Default::default(),
             dir_handler: Default::default(),
             panic_payload: None,
+            time_anchor: Instant::now(),
+            layouts,
+            threads: ThreadManager::default(),
+            static_roots: Vec::new(),
         }
     }
 }
 
 /// A rustc InterpCx for Miri.
-pub type MiriEvalContext<'mir, 'tcx> = InterpCx<'mir, 'tcx, Evaluator<'tcx>>;
+pub type MiriEvalContext<'mir, 'tcx> = InterpCx<'mir, 'tcx, Evaluator<'mir, 'tcx>>;
 
 /// A little trait that's useful to be inherited by extension traits.
 pub trait MiriEvalContextExt<'mir, 'tcx> {
@@ -194,8 +320,8 @@ impl<'mir, 'tcx> MiriEvalContextExt<'mir, 'tcx> for MiriEvalContext<'mir, 'tcx>
 }
 
 /// Machine hook implementations.
-impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
-    type MemoryKinds = MiriMemoryKind;
+impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'mir, 'tcx> {
+    type MemoryKind = MiriMemoryKind;
 
     type FrameExtra = FrameData<'tcx>;
     type MemoryExtra = MemoryExtra;
@@ -206,9 +332,17 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
     type MemoryMap =
         MonoHashMap<AllocId, (MemoryKind<MiriMemoryKind>, Allocation<Tag, Self::AllocExtra>)>;
 
-    const STATIC_KIND: Option<MiriMemoryKind> = Some(MiriMemoryKind::Static);
+    const GLOBAL_KIND: Option<MiriMemoryKind> = Some(MiriMemoryKind::Global);
 
-    const CHECK_ALIGN: bool = true;
+    #[inline(always)]
+    fn enforce_alignment(memory_extra: &MemoryExtra) -> bool {
+        memory_extra.check_alignment != AlignmentCheck::None
+    }
+
+    #[inline(always)]
+    fn force_int_for_alignment_check(memory_extra: &Self::MemoryExtra) -> bool {
+        memory_extra.check_alignment == AlignmentCheck::Int
+    }
 
     #[inline(always)]
     fn enforce_validity(ecx: &InterpCx<'mir, 'tcx, Self>) -> bool {
@@ -218,7 +352,6 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
     #[inline(always)]
     fn find_mir_or_eval_fn(
         ecx: &mut InterpCx<'mir, 'tcx, Self>,
-        _span: Span,
         instance: ty::Instance<'tcx>,
         args: &[OpTy<'tcx, Tag>],
         ret: Option<(PlaceTy<'tcx, Tag>, mir::BasicBlock)>,
@@ -241,23 +374,26 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
     #[inline(always)]
     fn call_intrinsic(
         ecx: &mut rustc_mir::interpret::InterpCx<'mir, 'tcx, Self>,
-        span: Span,
         instance: ty::Instance<'tcx>,
         args: &[OpTy<'tcx, Tag>],
         ret: Option<(PlaceTy<'tcx, Tag>, mir::BasicBlock)>,
         unwind: Option<mir::BasicBlock>,
     ) -> InterpResult<'tcx> {
-        ecx.call_intrinsic(span, instance, args, ret, unwind)
+        ecx.call_intrinsic(instance, args, ret, unwind)
     }
 
     #[inline(always)]
     fn assert_panic(
         ecx: &mut InterpCx<'mir, 'tcx, Self>,
-        span: Span,
         msg: &mir::AssertMessage<'tcx>,
         unwind: Option<mir::BasicBlock>,
     ) -> InterpResult<'tcx> {
-        ecx.assert_panic(span, msg, unwind)
+        ecx.assert_panic(msg, unwind)
+    }
+
+    #[inline(always)]
+    fn abort(_ecx: &mut InterpCx<'mir, 'tcx, Self>) -> InterpResult<'tcx, !> {
+        throw_machine_stop!(TerminationInfo::Abort(None))
     }
 
     #[inline(always)]
@@ -266,7 +402,7 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         bin_op: mir::BinOp,
         left: ImmTy<'tcx, Tag>,
         right: ImmTy<'tcx, Tag>,
-    ) -> InterpResult<'tcx, (Scalar<Tag>, bool, Ty<'tcx>)> {
+    ) -> InterpResult<'tcx, (Scalar<Tag>, bool, ty::Ty<'tcx>)> {
         ecx.binary_ptr_op(bin_op, left, right)
     }
 
@@ -278,10 +414,10 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         let layout = ecx.layout_of(dest.layout.ty.builtin_deref(false).unwrap().ty)?;
         // First argument: `size`.
         // (`0` is allowed here -- this is expected to be handled by the lang item).
-        let size = Scalar::from_uint(layout.size.bytes(), ecx.pointer_size());
+        let size = Scalar::from_machine_usize(layout.size.bytes(), ecx);
 
         // Second argument: `align`.
-        let align = Scalar::from_uint(layout.align.abi.bytes(), ecx.pointer_size());
+        let align = Scalar::from_machine_usize(layout.align.abi.bytes(), ecx);
 
         // Call the `exchange_malloc` lang item.
         let malloc = ecx.tcx.lang_items().exchange_malloc_fn().unwrap();
@@ -298,28 +434,26 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         Ok(())
     }
 
-    fn canonical_alloc_id(mem: &Memory<'mir, 'tcx, Self>, id: AllocId) -> AllocId {
-        let tcx = mem.tcx;
-        // Figure out if this is an extern static, and if yes, which one.
-        let def_id = match tcx.alloc_map.lock().get(id) {
-            Some(GlobalAlloc::Static(def_id)) if tcx.is_foreign_item(def_id) => def_id,
-            _ => {
-                // No need to canonicalize anything.
-                return id;
-            }
-        };
-        let attrs = tcx.get_attrs(def_id);
-        let link_name = match attr::first_attr_value_str_by_name(&attrs, sym::link_name) {
+    fn thread_local_static_alloc_id(
+        ecx: &mut InterpCx<'mir, 'tcx, Self>,
+        def_id: DefId,
+    ) -> InterpResult<'tcx, AllocId> {
+        ecx.get_or_create_thread_local_alloc_id(def_id)
+    }
+
+    fn extern_static_alloc_id(
+        memory: &Memory<'mir, 'tcx, Self>,
+        def_id: DefId,
+    ) -> InterpResult<'tcx, AllocId> {
+        let attrs = memory.tcx.get_attrs(def_id);
+        let link_name = match memory.tcx.sess.first_attr_value_str_by_name(&attrs, sym::link_name) {
             Some(name) => name,
-            None => tcx.item_name(def_id),
+            None => memory.tcx.item_name(def_id),
         };
-        // Check if we know this one.
-        if let Some(canonical_id) = mem.extra.extern_statics.get(&link_name) {
-            trace!("canonical_alloc_id: {:?} ({}) -> {:?}", id, link_name, canonical_id);
-            *canonical_id
+        if let Some(&id) = memory.extra.extern_statics.get(&link_name) {
+            Ok(id)
         } else {
-            // Return original id; `Memory::get_static_alloc` will throw an error.
-            id
+            throw_unsup_format!("`extern` static {:?} is not supported by Miri", def_id)
         }
     }
 
@@ -327,12 +461,16 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         memory_extra: &MemoryExtra,
         id: AllocId,
         alloc: Cow<'b, Allocation>,
-        kind: Option<MemoryKind<Self::MemoryKinds>>,
+        kind: Option<MemoryKind<Self::MemoryKind>>,
     ) -> (Cow<'b, Allocation<Self::PointerTag, Self::AllocExtra>>, Self::PointerTag) {
+        if Some(id) == memory_extra.tracked_alloc_id {
+            register_diagnostic(NonHaltingDiagnostic::CreatedAlloc(id));
+        }
+
         let kind = kind.expect("we set our STATIC_KIND so this cannot be None");
         let alloc = alloc.into_owned();
         let (stacks, base_tag) =
-            if let Some(stacked_borrows) = memory_extra.stacked_borrows.as_ref() {
+            if let Some(stacked_borrows) = &memory_extra.stacked_borrows {
                 let (stacks, base_tag) =
                     Stacks::new_allocation(id, alloc.size, Rc::clone(stacked_borrows), kind);
                 (Some(stacks), base_tag)
@@ -343,10 +481,10 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         let mut stacked_borrows = memory_extra.stacked_borrows.as_ref().map(|sb| sb.borrow_mut());
         let alloc: Allocation<Tag, Self::AllocExtra> = alloc.with_tags_and_extra(
             |alloc| {
-                if let Some(stacked_borrows) = stacked_borrows.as_mut() {
-                    // Only statics may already contain pointers at this point
-                    assert_eq!(kind, MiriMemoryKind::Static.into());
-                    stacked_borrows.static_base_ptr(alloc)
+                if let Some(stacked_borrows) = &mut stacked_borrows {
+                    // Only globals may already contain pointers at this point
+                    assert_eq!(kind, MiriMemoryKind::Global.into());
+                    stacked_borrows.global_base_ptr(alloc)
                 } else {
                     Tag::Untagged
                 }
@@ -357,9 +495,21 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
     }
 
     #[inline(always)]
-    fn tag_static_base_pointer(memory_extra: &MemoryExtra, id: AllocId) -> Self::PointerTag {
-        if let Some(stacked_borrows) = memory_extra.stacked_borrows.as_ref() {
-            stacked_borrows.borrow_mut().static_base_ptr(id)
+    fn before_deallocation(
+        memory_extra: &mut Self::MemoryExtra,
+        id: AllocId,
+    ) -> InterpResult<'tcx> {
+        if Some(id) == memory_extra.tracked_alloc_id {
+            register_diagnostic(NonHaltingDiagnostic::FreedAlloc(id));
+        }
+
+        Ok(())
+    }
+
+    #[inline(always)]
+    fn tag_global_base_pointer(memory_extra: &MemoryExtra, id: AllocId) -> Self::PointerTag {
+        if let Some(stacked_borrows) = &memory_extra.stacked_borrows {
+            stacked_borrows.borrow_mut().global_base_ptr(id)
         } else {
             Tag::Untagged
         }
@@ -371,30 +521,54 @@ impl<'mir, 'tcx> Machine<'mir, 'tcx> for Evaluator<'tcx> {
         kind: mir::RetagKind,
         place: PlaceTy<'tcx, Tag>,
     ) -> InterpResult<'tcx> {
-        if ecx.memory.extra.stacked_borrows.is_none() {
-            // No tracking.
-            Ok(())
-        } else {
+        if ecx.memory.extra.stacked_borrows.is_some() {
             ecx.retag(kind, place)
+        } else {
+            Ok(())
         }
     }
 
     #[inline(always)]
-    fn stack_push(ecx: &mut InterpCx<'mir, 'tcx, Self>) -> InterpResult<'tcx, FrameData<'tcx>> {
+    fn init_frame_extra(
+        ecx: &mut InterpCx<'mir, 'tcx, Self>,
+        frame: Frame<'mir, 'tcx, Tag>,
+    ) -> InterpResult<'tcx, Frame<'mir, 'tcx, Tag, FrameData<'tcx>>> {
         let stacked_borrows = ecx.memory.extra.stacked_borrows.as_ref();
         let call_id = stacked_borrows.map_or(NonZeroU64::new(1).unwrap(), |stacked_borrows| {
             stacked_borrows.borrow_mut().new_call()
         });
-        Ok(FrameData { call_id, catch_panic: None })
+        let extra = FrameData { call_id, catch_unwind: None };
+        Ok(frame.with_extra(extra))
+    }
+
+    fn stack<'a>(
+        ecx: &'a InterpCx<'mir, 'tcx, Self>
+    ) -> &'a [Frame<'mir, 'tcx, Self::PointerTag, Self::FrameExtra>] {
+        ecx.active_thread_stack()
+    }
+
+    fn stack_mut<'a>(
+        ecx: &'a mut InterpCx<'mir, 'tcx, Self>
+    ) -> &'a mut Vec<Frame<'mir, 'tcx, Self::PointerTag, Self::FrameExtra>> {
+        ecx.active_thread_stack_mut()
     }
 
     #[inline(always)]
-    fn stack_pop(
+    fn after_stack_push(ecx: &mut InterpCx<'mir, 'tcx, Self>) -> InterpResult<'tcx> {
+        if ecx.memory.extra.stacked_borrows.is_some() {
+            ecx.retag_return_place()
+        } else {
+            Ok(())
+        }
+    }
+
+    #[inline(always)]
+    fn after_stack_pop(
         ecx: &mut InterpCx<'mir, 'tcx, Self>,
-        extra: FrameData<'tcx>,
+        frame: Frame<'mir, 'tcx, Tag, FrameData<'tcx>>,
         unwinding: bool,
-    ) -> InterpResult<'tcx, StackPopInfo> {
-        ecx.handle_stack_pop(extra, unwinding)
+    ) -> InterpResult<'tcx, StackPopJump> {
+        ecx.handle_stack_pop(frame.extra, unwinding)
     }
 
     #[inline(always)]
@@ -421,7 +595,7 @@ impl AllocationExtra<Tag> for AllocExtra {
         ptr: Pointer<Tag>,
         size: Size,
     ) -> InterpResult<'tcx> {
-        if let Some(ref stacked_borrows) = alloc.extra.stacked_borrows {
+        if let Some(stacked_borrows) = &alloc.extra.stacked_borrows {
             stacked_borrows.memory_read(ptr, size)
         } else {
             Ok(())
@@ -434,7 +608,7 @@ impl AllocationExtra<Tag> for AllocExtra {
         ptr: Pointer<Tag>,
         size: Size,
     ) -> InterpResult<'tcx> {
-        if let Some(ref mut stacked_borrows) = alloc.extra.stacked_borrows {
+        if let Some(stacked_borrows) = &mut alloc.extra.stacked_borrows {
             stacked_borrows.memory_written(ptr, size)
         } else {
             Ok(())
@@ -447,21 +621,10 @@ impl AllocationExtra<Tag> for AllocExtra {
         ptr: Pointer<Tag>,
         size: Size,
     ) -> InterpResult<'tcx> {
-        if let Some(ref mut stacked_borrows) = alloc.extra.stacked_borrows {
+        if let Some(stacked_borrows) = &mut alloc.extra.stacked_borrows {
             stacked_borrows.memory_deallocated(ptr, size)
         } else {
             Ok(())
         }
     }
 }
-
-impl MayLeak for MiriMemoryKind {
-    #[inline(always)]
-    fn may_leak(self) -> bool {
-        use self::MiriMemoryKind::*;
-        match self {
-            Rust | C | WinHeap => false,
-            Machine | Static => true,
-        }
-    }
-}
